/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright © 2003 Atheros Communications, Inc.,  All Rights Reserved.
 */


/*******************************************************************************
 * ar531x_proc provides a channel for communication between user-level
 * and this AR531X BSP.  Communication is through files in /proc/ar531x.
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/gdb-stub.h>

#include "ar531xlnx.h"

#ifdef CONFIG_PROC_FS


#ifdef CONFIG_KGDB
/*
 * Start kgdb debugging: "echo 1 > /proc/ar531x/debug".
 * If you need kgdb early in the boot process, use CONFIG_EARLY_STOP.
 */
static int
ar531x_debug_write(struct file *file, const char *buf, unsigned long count, void *data)
{
    kgdbInit();
}
#endif /* CONFIG_KGDB */

#ifdef CONFIG_AR531X_WATCHDOG
static int
ar531x_wd_read(char *page, char **start, off_t off,
               int count, int *eof, void *data)
{
    return sprintf(page, "%d\n", 
                   watchdog_is_enabled() ? watchdog_min_timer_reached() : 0);
}

/*
 * Enable/Disable system watchdog timer via /proc.
 * Example: echo 4000 > /proc/ar531x/watchdog
 */
static int
ar531x_wd_write(struct file *file, const char *buf, unsigned long count, void *data)
{
    int watchdog_milliseconds;

    if (sscanf(buf, "%d", &watchdog_milliseconds) != 1)
        return -EINVAL;

    if (watchdog_milliseconds) {
        /* If watchdog is not disabled by board config, then start it */
        if (ar531x_board_configuration &&
            !(ar531x_board_configuration->config & BD_DISWATCHDOG)) {
            watchdog_start(watchdog_milliseconds);
        }
    } else {
        watchdog_stop();
    }

    return count;
}
#endif /* CONFIG_AR531X_WATCHDOG */

/*******************************************************************************
* ar531x_proc_setup - Initialize the /proc files needed for AR531X.
*/
int
ar531x_proc_setup(void)
{
    struct proc_dir_entry *ar531x_proc_entry;

    ar531x_proc_entry = proc_mkdir("ar531x", NULL);
    if (!ar531x_proc_entry) {
        return -ENOENT;
    }

#ifdef CONFIG_KGDB
    {
        struct proc_dir_entry *ar531x_debug_entry;
        ar531x_debug_entry = create_proc_entry("debug", 0644, ar531x_proc_entry);
        if (ar531x_debug_entry) {
            ar531x_debug_entry->write_proc = ar531x_debug_write;
        }
    }
#endif

#ifdef CONFIG_AR531X_WATCHDOG
    {
        struct proc_dir_entry *ar531x_wd_entry;

        ar531x_wd_entry = create_proc_entry("watchdog", 0644, ar531x_proc_entry);
        if (ar531x_wd_entry) {
            ar531x_wd_entry->write_proc = ar531x_wd_write;
            ar531x_wd_entry->read_proc = ar531x_wd_read;
        }
    }
#endif

    return 0;
}

module_init(ar531x_proc_setup);

#endif /* CONFIG_PROC_FS */
