/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright © 2003 Atheros Communications, Inc.,  All Rights Reserved.
 */

/*******************************************************************************
                        *****  Under Construction! *****
                        *****  Under Construction! *****
                        *****  Under Construction! *****
 ******************************************************************************/

#include <linux/config.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <asm/io.h>

#include "ar531xlnx.h"
#include "asm/atheros/wmac_support.h"

/*
 * The WMAC Support Layer provides services that are system-dependent,
 * but which may be needed by WMAC implementations.  The WMACSL hides
 * the details of how hardware is *connected* to the system. So, for
 * instance, there is one WMACSL implementation suitable for
 * PCI-connected AR5212 WMAC, and a different WMACSL implementation
 * (but with the same API) for AR5312-connected AR5212 WMAC.
 *
 * Examples of WMAC Support Layer functions:
 *   read/write "EEprom" (may be emulated in flash)
 *   Reset the WMAC hardware
 *   Get WMAC revision information
 */


/*************START GENERIC WMACSL INTERFACE***********/

/*
 * This is the portion of a WMACsl_id_s that is shared by all WMACsl
 * implementations.  It is initialized by the *_WMACsl_init interface.
 * 
 * TBDXXX: If we ever have a PCI WMACsl, move this definition and the
 * generic implementations to a shared file.  Leave the ar5312-specific
 * implementations here.
 */
struct WMACsl_id_common_s {
	unsigned int (*WMACsl_LineSizeGet)(WMACsl_id_t *id);
	void         (*WMACsl_Reset)(WMACsl_id_t *wmacsl_id, unsigned int RCMask);
	void         (*WMACsl_UnReset)(WMACsl_id_t *wmacsl_id);
	void         (*WMACsl_EepromRead)(WMACsl_id_t *wmacsl_id,
                                          unsigned int off,
                                          unsigned int nbytes,
                                          char *data);
	void         (*WMACsl_EepromWrite)(WMACsl_id_t *wmacsl_id,
                                           unsigned int off,
                                           unsigned int nbytes,
                                           char *data);
	void         (*WMACsl_GpioCfgOutput)(WMACsl_id_t *wmacsl_id, int gpio);
	void         (*WMACsl_GpioCfgInput)(WMACsl_id_t *wmacsl_id, int gpio);
	void         (*WMACsl_GpioCfgIntr)(WMACsl_id_t *wmacsl_id,
                                          int gpio,
                                          int ilevel);
	void         (*WMACsl_GpioSet)(WMACsl_id_t *wmacsl_id,
                                      int gpio,
                                      int val);
	int          (*WMACsl_GpioGet)(WMACsl_id_t *wmacsl_id,
                                       int gpio);
	WMACsl_dma_addr_t (*WMACsl_MapSingle)(WMACsl_id_t *wmacsl_id,
                                           void *data,
                                           unsigned int length,
                                           enum WMACslDmaDirection direc);
	void         (*WMACsl_UnmapSingle)(WMACsl_id_t *wmacsl_id,
                                           WMACsl_dma_addr_t dma_addr,
                                           unsigned int length,
                                           enum WMACslDmaDirection direc);
	void         (*WMACsl_DmaSyncSingle)(WMACsl_id_t *wmacsl_id,
                                             WMACsl_dma_addr_t dma_addr,
                                             unsigned int length,
                                             enum WMACslDmaDirection direc);
	void        *(*WMACsl_AllocConsistent)(WMACsl_id_t *wmacsl_id,
                                               unsigned int length,
                                               WMACsl_dma_addr_t *dma_addrp);
	void         (*WMACsl_FreeConsistent)(WMACsl_id_t *wmacsl_id,
                                              unsigned int length,
                                              void *desc,
                                              WMACsl_dma_addr_t dma_addr);
	int          (*WMACsl_StateIsSleeping)(WMACsl_id_t *wmacsl_id);
	enum WMACslPowerMode (*WMACsl_PowerModeGet)(WMACsl_id_t *wmacsl_id);
	void         (*WMACsl_DMAIn)(WMACsl_id_t *wmacsl_id,
                                     void *data,
                                     unsigned int length);
	void         (*WMACsl_WMACRevInfoGet)(WMACsl_id_t *wmacsl_id,
                                              unsigned int *WMACVersion,
                                              unsigned int *WMACRevision);
	char *       (*WMACsl_mac_address_get)(WMACsl_id_t *wmacsl_id);
	void *       (*WMACsl_MemStartGet)(WMACsl_id_t *wmacsl_id);
	unsigned long (*WMACsl_MemSizeGet)(WMACsl_id_t *wmacsl_id);
};

unsigned int
WMACsl_LineSizeGet(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_LineSizeGet(wmacsl_id);
}

void
WMACsl_Reset(WMACsl_id_t *wmacsl_id,
             unsigned int RCMask)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_Reset(wmacsl_id, RCMask);
}

void
WMACsl_UnReset(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_UnReset(wmacsl_id);
}

void
WMACsl_EepromRead(WMACsl_id_t *wmacsl_id,
                  unsigned int off,
                  unsigned int nbytes,
                  char *data)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_EepromRead(wmacsl_id, off, nbytes, data);
}

void
WMACsl_EepromWrite(WMACsl_id_t *wmacsl_id,
                  unsigned int off,
                  unsigned int nbytes,
                  char *data)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_EepromWrite(wmacsl_id, off, nbytes, data);
}

void
WMACsl_GpioCfgOutput(WMACsl_id_t *wmacsl_id,
                     int gpio)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_GpioCfgOutput(wmacsl_id, gpio);
}

void
WMACsl_GpioCfgInput(WMACsl_id_t *wmacsl_id,
                    int gpio)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_GpioCfgInput(wmacsl_id, gpio);
}

void
WMACsl_GpioCfgIntr(WMACsl_id_t *wmacsl_id,
                   int gpio,
                   int ilevel)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_GpioCfgIntr(wmacsl_id, gpio, ilevel);
}

void
WMACsl_GpioSet(WMACsl_id_t *wmacsl_id,
               int gpio,
               int val)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_GpioSet(wmacsl_id, gpio, val);
}

int
WMACsl_GpioGet(WMACsl_id_t *wmacsl_id,
               int gpio)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_GpioGet(wmacsl_id, gpio);
}


WMACsl_dma_addr_t
WMACsl_MapSingle(WMACsl_id_t *wmacsl_id,
                 void *data,
                 unsigned int length,
                 enum WMACslDmaDirection direc)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_MapSingle(wmacsl_id, data, length, direc);
}

void
WMACsl_UnmapSingle(WMACsl_id_t *wmacsl_id,
                   WMACsl_dma_addr_t dma_addr,
                   unsigned int length,
                   enum WMACslDmaDirection direc)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_UnmapSingle(wmacsl_id, dma_addr, length, direc);
}

void
WMACsl_DmaSyncSingle(WMACsl_id_t *wmacsl_id,
                     WMACsl_dma_addr_t dma_addr,
                     unsigned int length,
                     enum WMACslDmaDirection direc)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_DmaSyncSingle(wmacsl_id, dma_addr, length, direc);
}

void *
WMACsl_AllocConsistent(WMACsl_id_t *wmacsl_id,
                       unsigned int length,
                       WMACsl_dma_addr_t *dma_addrp)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_AllocConsistent(wmacsl_id, length, dma_addrp);
}

void
WMACsl_FreeConsistent(WMACsl_id_t *wmacsl_id,
                      unsigned int length,
                      void *desc,
                      WMACsl_dma_addr_t dma_addr)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_FreeConsistent(wmacsl_id, length, desc, dma_addr);
}

int
WMACsl_StateIsSleeping(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_StateIsSleeping(wmacsl_id);
}

enum WMACslPowerMode
WMACsl_PowerModeGet(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_PowerModeGet(wmacsl_id);
}

void
WMACsl_DMAIn(WMACsl_id_t *wmacsl_id,
             void *data,
             unsigned int length)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_DMAIn(wmacsl_id, data, length);
}

char *
WMACsl_mac_address_get(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_mac_address_get(wmacsl_id);
}

void *
WMACsl_MemStartGet(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_MemStartGet(wmacsl_id);
}

unsigned long
WMACsl_MemSizeGet(WMACsl_id_t *wmacsl_id)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	return wmacsl_cmn->WMACsl_MemSizeGet(wmacsl_id);
}

void
WMACsl_WMACRevInfoGet(WMACsl_id_t *wmacsl_id,
                             unsigned int *WMACVersion,
                             unsigned int *WMACRevision)
{
	struct WMACsl_id_common_s *wmacsl_cmn;
	wmacsl_cmn = (struct WMACsl_id_common_s *)wmacsl_id;

	wmacsl_cmn->WMACsl_WMACRevInfoGet(wmacsl_id, WMACVersion, WMACRevision);
}

/*************END GENERIC WMACSL INTERFACE***********/


/*
 * There is one WMAC Support Layer (WMACsl) structure for
 * each WMAC.  It contains all of the system-dependent
 * information needed to deal with a WMAC.
 */
struct ar5312_WMACsl {
    struct WMACsl_id_common_s	wmacsl_id_cmn; /* Must be first */

    int           initialized;
    int           wmac_idx;
    unsigned int  irq;
    unsigned long mem_start;
    unsigned long mem_size;
    UINT32        ar531x_reset_bits;
    UINT32        ar531x_enable_bits;
    UINT32        wmac_resetBB_bits;
    UINT32        wmac_resetMAC_bits;
    char         *EepromStartAddr;
};

/* AR5312 Wireless MAC hardware information */
struct ar5312_WMACsl ar5312_wmac_hwinfo[AR531X_NUM_WMAC] = {
    {
        initialized:        0,
        wmac_idx:            0,
        irq:                AR531X_IRQ_WLAN0_INTRS,
        mem_start:          PHYS_TO_K1(AR531X_WLAN0),
        mem_size:           0x00500000,
        ar531x_reset_bits:  AR531X_RESET_WLAN0,
        ar531x_enable_bits: AR531X_ENABLE_WLAN0,
        wmac_resetBB_bits:  AR531X_RESET_WARM_WLAN0_BB,
        wmac_resetMAC_bits: AR531X_RESET_WLAN0 | AR531X_RESET_WARM_WLAN0_MAC,
        EepromStartAddr:    NULL,
    },

    {
        initialized:        0,
        wmac_idx:            1,
        irq:                AR531X_IRQ_WLAN1_INTRS,
        mem_start:          PHYS_TO_K1(AR531X_WLAN1),
        mem_size:           0x00500000,
        ar531x_reset_bits:  AR531X_RESET_WLAN1,
        ar531x_enable_bits: AR531X_ENABLE_WLAN1,
        wmac_resetBB_bits:  AR531X_RESET_WLAN1,
        wmac_resetMAC_bits: AR531X_RESET_WARM_WLAN1_MAC | AR531X_RESET_WARM_WLAN1_BB,
        EepromStartAddr:    NULL,
    },
};


/***********************************************************/
/* AR5312-specific implementations of the WMACSL interface */
/***********************************************************/

/*
 * Returns 1 if the specified WMAC is enabled; 0 otherwise.
 */
int
ar5312_WMACsl_UnitIsValid(unsigned int wmac_idx)
{
    struct ar531x_boarddata *board_config_data =
        (struct ar531x_boarddata *)ar531x_board_configuration;

    unsigned int config;

    if (board_config_data == NULL)
        return 0;

    config = board_config_data->config;

    if ((wmac_idx == 0) && (config & BD_WLAN0))
        return 1;

    if ((wmac_idx == 1) && (config & BD_WLAN1))
        return 1;

    return 0; /* invalid */

}

unsigned int
ar5312_WMACsl_LineSizeGet(WMACsl_id_t *id)
{
    return 16; /* bytes */
}

/*
 * Accessor functions for data associated with a WMAC.
 */
unsigned int
ar5312_WMACsl_IrqGet(WMACsl_id_t *id)
{
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    return wmacsl_info->irq;
}

void *
ar5312_WMACsl_MemStartGet(WMACsl_id_t *id)
{
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    return (void *)wmacsl_info->mem_start;
}

unsigned long
ar5312_WMACsl_MemSizeGet(WMACsl_id_t *id)
{
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    return wmacsl_info->mem_size;
}

/*******************************************************************************
* ar5312_WMACsl_UnReset pulls a WMAC OUT of reset.
* Intended for use at startup.
*/
void
ar5312_WMACsl_UnReset(WMACsl_id_t *id)
{
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    sysRegWrite(AR531X_RESET,
                (sysRegRead(AR531X_RESET) &
               ~(wmacsl_info->ar531x_reset_bits)));
}

/*******************************************************************************
* ar5312_WMACsl_Enable enables a WMAC to do DMA.
*/
void
ar5312_WMACsl_Enable(WMACsl_id_t *id)
{
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    sysRegWrite(AR531X_ENABLE,
                (sysRegRead(AR531X_ENABLE) |
                (wmacsl_info->ar531x_enable_bits)));
}

/*******************************************************************************
* ar5312_WMACsl_WMACRevInfoGet returns the WMAC version and revision information.
*/
void
ar5312_WMACsl_WMACRevInfoGet(WMACsl_id_t *id,
                             unsigned int *WMACVersion,
                             unsigned int *WMACRevision)
{
    UINT32 val;

    val = sysRegRead(AR531X_REV);
    *WMACVersion = (val & AR531X_REV_WMAC_MAJ) >> AR531X_REV_WMAC_MAJ_S;
    *WMACRevision = (val & AR531X_REV_WMAC_MIN) >> AR531X_REV_WMAC_MIN_S;
}

/*******************************************************************************
* ar5312_WMACsl_SOCRevInfoGet returns the WiSOC version and revision information.
*/
void
ar5312_WMACsl_SOCRevInfoGet(unsigned int *version,
                            unsigned int *revision)
{
    UINT32 val;

    val = sysRegRead(AR531X_REV);
    *version = (val & AR531X_REV_MAJ) >> AR531X_REV_MAJ_S;
    *revision = (val & AR531X_REV_MIN) >> AR531X_REV_MIN_S;
}

/*******************************************************************************
* ar5312_WMACsl_Reset resets (and then UNresets) the specified
* wireless components.
*/
void
ar5312_WMACsl_Reset(WMACsl_id_t *id,
                    unsigned int RCMask)
{
    UINT32 resetBB, resetBits, regMask;
    UINT32 reg;
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    resetBB = wmacsl_info->wmac_resetBB_bits;
    resetBits = wmacsl_info->wmac_resetMAC_bits;
    regMask = ~(resetBB | resetBits);

    reg = sysRegRead(AR531X_RESET);             /* read before */

    if (RCMask == AR_RC_BB) {
        /* Put baseband in reset */
        reg |= resetBB;
        sysRegWrite(AR531X_RESET, reg);
        (void)sysRegRead(AR531X_RESET);         /* read after */
        return;
    }

    if (RCMask) {
        /*
         * Reset the MAC and baseband.  This is a bit different than
         * the PCI version, but holding in reset causes problems.
         */
        reg &= regMask;
        reg |= resetBits;
        sysRegWrite(AR531X_RESET, reg);
        (void)sysRegRead(AR531X_RESET);         /* read after */
        udelay(100);
    }

    /* Bring MAC and baseband out of reset */
    reg &= regMask;
    (void)sysRegRead(AR531X_RESET);             /* read before */
    sysRegWrite(AR531X_RESET, reg);
    (void)sysRegRead(AR531X_RESET);             /* read after */
}

/*******************************************************************************
* ar5312_WMACsl_EepromInit sets up for future EepromRead/EepromWrite.
*/
static void
ar5312_WMACsl_EepromInit(WMACsl_id_t *id)
{
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;

    wmacsl_info->EepromStartAddr = ar531x_radio_configuration;
}

/*******************************************************************************
* ar5312_WMACsl_EepromRead reads nbytes of data from offset "off" in the
* emulated Eeprom.  Stores the results in "data".
*/
void
ar5312_WMACsl_EepromRead(WMACsl_id_t *id,
                         unsigned int off,
                         unsigned int nbytes,
                         char *data)
{
    int i;
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;
    char *EepromAddr = wmacsl_info->EepromStartAddr;

    for (i=0; i<nbytes; i++, off++)
        data[i] = EepromAddr[off];
}

/*******************************************************************************
* ar5312_WMACsl_EepromWrite writes nbytes of "data" to offset "off" in the
* emulated Eeprom.
*/
void
ar5312_WMACsl_EepromWrite(WMACsl_id_t *id,
                          unsigned int off,
                          unsigned int nbytes,
                          char *data)
{
    printk("ar5312_WMACsl_EepromWrite not implemented. off=0x%8.8x nbytes=%d\n",
           off, nbytes);
}

/*******************************************************************************
* The GPIO wrappers are provided mostly just to make obvious
* which functions are provided by the BSP to wireless implementations.
*/
void
ar5312_WMACsl_GpioCfgOutput(WMACsl_id_t *id,
                            int gpio)
{
    ar531x_gpio_ctrl_output(gpio);
}

void
ar5312_WMACsl_GpioCfgInput(WMACsl_id_t *id,
                           int gpio)
{
    ar531x_gpio_ctrl_input(gpio);
}

void
ar5312_WMACsl_GpioSet(WMACsl_id_t *id,
                      int gpio,
                      int val)
{
    ar531x_gpio_set(gpio, val);
}

int
ar5312_WMACsl_GpioGet(WMACsl_id_t *id,
                      int gpio)
{
    return ar531x_gpio_get(gpio);
}

void
ar5312_WMACsl_GpioCfgIntr(WMACsl_id_t *id,
                          int gpio,
                          int ilevel)
{
    ASSERT(ilevel);
    ar531x_gpio_intr_enable(gpio + AR531X_GPIO_IRQ_BASE);
}


/*
 * Address space support is simple on AR5312, since
 * we don't use mapped addresses.  All virtual addresses
 * are either KSEG0 (cached, unmapped) or KSEG1 (uncached,
 * unmapped).
 */

WMACsl_dma_addr_t
ar5312_WMACsl_MapSingle(WMACsl_id_t *id,
                        void *data,
                        unsigned int length,
                        enum WMACslDmaDirection direc)
{
    ASSERT(IS_UNMAPPED_VADDR(data));

    return UNMAPPED_TO_PHYS(data);
}

void
ar5312_WMACsl_UnmapSingle(WMACsl_id_t *id,
                          WMACsl_dma_addr_t dma_addr,
                          unsigned int length,
                          enum WMACslDmaDirection direc)
{
    /* do nothing */
}

void
ar5312_WMACsl_DmaSyncSingle(WMACsl_id_t *id,
                            WMACsl_dma_addr_t dma_addr,
                            unsigned int length,
                            enum WMACslDmaDirection direc)
{
    /*
     * We use a write-through cache, so we don't need to do anything
     * for WMACSL_DMA_TO_DEVICE.  For WMACSL_IO_DMA_FROM_DEVICE,
     * we need to invalidate the data cache.
     */
    if (direc == WMACSL_DMA_FROM_DEVICE) {
        A_DATA_CACHE_INVAL(PHYS_TO_K0(dma_addr), length);
    }
}

void *
ar5312_WMACsl_AllocConsistent(WMACsl_id_t *id,
                              unsigned int length,
                              WMACsl_dma_addr_t *dma_addrp)
{
    void *addr = MALLOC(length);
    ASSERT(IS_UNMAPPED_VADDR(addr));

    /* Convert from unmapped virtual address to physical address */
    addr = (void *)UNMAPPED_TO_PHYS(addr);

    *dma_addrp = (WMACsl_dma_addr_t)addr;

    return (void *)PHYS_TO_K1(addr); /* return virtual unmapped uncached */
}

void
ar5312_WMACsl_FreeConsistent(WMACsl_id_t *id,
                             unsigned int length,
                             void *desc,
                             WMACsl_dma_addr_t dma_addr)
{
    /* do nothing */
}

int
ar5312_WMACsl_StateIsSleeping(WMACsl_id_t *id)
{
    return 0;
}

enum WMACslPowerMode
ar5312_WMACsl_PowerModeGet(WMACsl_id_t *id)
{
    return WMACSL_POWER_MODE_NORMAL;
}

void
ar5312_WMACsl_DMAIn(WMACsl_id_t *id,
                    void *data,
                    unsigned int length)
{
    A_DATA_CACHE_INVAL(data, length);
}

/*
 * Fetch a pointer to a WLAN's MAC address
 * in the Board Configuration data (in flash).
 */
char *
ar5312_WMACsl_mac_address_get(WMACsl_id_t *id)
{
    struct ar531x_boarddata *board_config_data =
        (struct ar531x_boarddata *)ar531x_board_configuration;
    struct ar5312_WMACsl *wmacsl_info = (struct ar5312_WMACsl *)id;
    unsigned int wmac_idx = wmacsl_info->wmac_idx;

    if (board_config_data == NULL)
        return NULL;

    if (wmac_idx == 0) {
        return board_config_data->wlan0Mac;
    }

    if (wmac_idx == 1) {
        return board_config_data->wlan1Mac;
    }

    printk("Invalid WLAN wmac index (%d)!\n", wmac_idx);
    return NULL;
}

WMACsl_id_t *
ar5312_WMACsl_init(unsigned int wmac_idx)
{
    struct ar5312_WMACsl *wmacsl_info = &ar5312_wmac_hwinfo[wmac_idx];

    wmacsl_info->wmac_idx = wmac_idx;

    wmacsl_info->wmacsl_id_cmn.WMACsl_LineSizeGet     =
	    ar5312_WMACsl_LineSizeGet;

    wmacsl_info->wmacsl_id_cmn.WMACsl_Reset           =
	    ar5312_WMACsl_Reset;

    wmacsl_info->wmacsl_id_cmn.WMACsl_UnReset         =
	    ar5312_WMACsl_UnReset;

    wmacsl_info->wmacsl_id_cmn.WMACsl_EepromRead      =
	    ar5312_WMACsl_EepromRead;

    wmacsl_info->wmacsl_id_cmn.WMACsl_EepromWrite     =
	    ar5312_WMACsl_EepromWrite;

    wmacsl_info->wmacsl_id_cmn.WMACsl_GpioCfgOutput   =
	    ar5312_WMACsl_GpioCfgOutput;

    wmacsl_info->wmacsl_id_cmn.WMACsl_GpioCfgInput    =
	    ar5312_WMACsl_GpioCfgInput;

    wmacsl_info->wmacsl_id_cmn.WMACsl_GpioCfgIntr     =
	    ar5312_WMACsl_GpioCfgIntr;

    wmacsl_info->wmacsl_id_cmn.WMACsl_GpioSet         =
	    ar5312_WMACsl_GpioSet;

    wmacsl_info->wmacsl_id_cmn.WMACsl_GpioGet         =
	    ar5312_WMACsl_GpioGet;

    wmacsl_info->wmacsl_id_cmn.WMACsl_MapSingle       =
	    ar5312_WMACsl_MapSingle;

    wmacsl_info->wmacsl_id_cmn.WMACsl_UnmapSingle     =
	    ar5312_WMACsl_UnmapSingle;

    wmacsl_info->wmacsl_id_cmn.WMACsl_DmaSyncSingle   =
	    ar5312_WMACsl_DmaSyncSingle;

    wmacsl_info->wmacsl_id_cmn.WMACsl_AllocConsistent =
	    ar5312_WMACsl_AllocConsistent;

    wmacsl_info->wmacsl_id_cmn.WMACsl_FreeConsistent  =
	    ar5312_WMACsl_FreeConsistent;

    wmacsl_info->wmacsl_id_cmn.WMACsl_StateIsSleeping =
	    ar5312_WMACsl_StateIsSleeping;

    wmacsl_info->wmacsl_id_cmn.WMACsl_PowerModeGet    =
	    ar5312_WMACsl_PowerModeGet;

    wmacsl_info->wmacsl_id_cmn.WMACsl_DMAIn           =
	    ar5312_WMACsl_DMAIn;

    wmacsl_info->wmacsl_id_cmn.WMACsl_WMACRevInfoGet  =
	    ar5312_WMACsl_WMACRevInfoGet;

    wmacsl_info->wmacsl_id_cmn.WMACsl_mac_address_get =
	    ar5312_WMACsl_mac_address_get;

    wmacsl_info->wmacsl_id_cmn.WMACsl_MemStartGet     =
	    ar5312_WMACsl_MemStartGet;

    wmacsl_info->wmacsl_id_cmn.WMACsl_MemSizeGet      =
	    ar5312_WMACsl_MemSizeGet;



    wmacsl_info->initialized = 1;

    ar5312_WMACsl_EepromInit((WMACsl_id_t *)wmacsl_info);

    return (WMACsl_id_t *)wmacsl_info;
}

