/* $Id: engine.c,v 1.2 2004/01/19 22:31:30 jgordon Exp $ */

/* 
 * $Log: engine.c,v $
 * Revision 1.2  2004/01/19 22:31:30  jgordon
 * merge from GATEWAY_PRODUCT_1_BRANCH (MAINLINE_MP_1) to mainline
 *
 * Revision 1.1.2.1  2004/01/06 01:52:30  alan
 * Threaded interpreter engine
 *
 * Revision 1.1.2.1.2.2.2.4  1999/11/21 16:51:13  helmut
 * minor changes
 *
 * Revision 1.1.2.1.2.3  1999/11/21 16:39:24  helmut
 * minor changes
 *
 * Revision 1.1.2.1.2.2.2.3  1999/11/15 23:38:14  helmut
 * use Instr union in place of casts to increase portability.
 *
 * Revision 1.1.2.1.2.2.2.2  1999/11/15 17:20:17  helmut
 * some checks disabled
 *
 * Revision 1.1.2.1.2.2.2.1  1999/11/14 20:09:08  helmut
 * quick instructions for ldc, invoke and putfield.
 *
 * Revision 1.1.2.1.2.2  1999/11/11 18:50:33  helmut
 * *** empty log message ***
 *
 * Revision 1.1.2.1.2.1  1999/11/10 11:49:20  helmut
 * *** empty log message ***
 *
 * Revision 1.1.2.1  1999/10/31 18:21:14  helmut
 * First checkin.
 *
 */

#include "debug.h"
#define	CDBG(s) 	DBG(INT_VMCALL, s)
#define	RDBG(s) 	DBG(INT_RETURN, s)
#define	NDBG(s) 	DBG(INT_NATIVE, s)
#define	IDBG(s)		DBG(INT_INSTR, s)
#define	CHDBG(s)	DBG(INT_CHECKS, s)

/*  #ifndef DEBUG
#define dprintf printf
#endif */

#include "config.h"
#include "config-std.h"
#include "config-math.h"
#include "config-mem.h"
#include "config-setjmp.h"
#include "classMethod.h"
#include "gtypes.h"
#include "bytecode.h"
#include "slots.h"
#include "access.h"
#include "object.h"
#include "constants.h"
#include "gc.h"
#include "lookup.h"
#include "code-analyse.h"
#include "soft.h"
#include "exception.h"
#include "external.h"
#include "baseClasses.h"
#include "thread.h"
#include "jthread.h"
#include "locks.h"
#include "checks.h"
#include "errors.h"
#include "md.h"

#include "icode.h"
#include "engine.h"
#include "checks.h"
#include "machine.h"
#include "translator.h"



/* Define CREATE_NULLPOINTER_CHECKS in md.h when your machine cannot use the
 * MMU for detecting null pointer accesses */
#if defined(CREATE_NULLPOINTER_CHECKS)
#define CHECK_NULL(_i, _s, _n)                                  \
      cbranch_ref_const_ne((_s), 0, reference_label(_i, _n)); \
      softcall_nullpointer();                                 \
      set_label(_i, _n)
#else
#define CHECK_NULL(_i, _s, _n)
#endif

#if defined(KAFFE_PROFILER)
int profFlag;			 /* flag to control profiling */
#endif



static  void engine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *retval, volatile VmExceptHandler *mjbuf, Hjava_lang_Thread *tid,
		   Instr **labels);

void virtualMachine (struct _methods* volatile meth, slots* volatile arg, slots* volatile retval, Hjava_lang_Thread* volatile tid) {
	methods *volatile const vmeth = meth;
	Hjava_lang_Object* volatile mobj;
	VmExceptHandler mjbuf;
	accessFlags methaccflags;

	struct _slots *volatile lcl;
	struct _slots *volatile sp;
	uintp volatile npc;

	int32 idx;
	jint low;

	errorInfo einfo;
	Hjava_lang_Throwable* overflow;
	jint *needOnStack;


	if (tid != 0)  {
		/* implement stack overflow check */
		needOnStack = &unhand(tid)->needOnStack;

		if (jthread_stackcheck(*needOnStack) == false) {
			overflow = (Hjava_lang_Throwable*)
				unhand(tid)->stackOverflowError;
			if (overflow != 0) {
				if (*needOnStack == STACK_LOW) {
					fprintf(stderr, 
					    "Panic: unhandled StackOverflowError()\n");
					ABORT();
				}
				*needOnStack = STACK_LOW;
				throwException(overflow);
			}
		}
	}

CDBG(	dprintf("Call: %s.%s%s.\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth)); )

	/* If this is native, then call the real function */
	methaccflags = meth->accflags;
	if (methaccflags & ACC_NATIVE) {
NDBG(		dprintf("Call to native %s.%s%s.\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth)); )
		if (methaccflags & ACC_STATIC) {
			callMethodA(meth, meth, 0, (jvalue*)arg, (jvalue*)retval, 1);
		}
		else {
			callMethodA(meth, meth, ((jvalue*)arg)[0].l, &((jvalue*)arg)[1], (jvalue*)retval, 1);
		}
		return;
	}

	/* Verify method if required */
	if ((methaccflags & ACC_VERIFIED) == 0) {
/*		codeinfo* codeInfo;
		bool success = verifyMethod(meth, &codeInfo, &einfo);
		tidyVerifyMethod(&codeInfo);
		if (success == false) {
			throwError(&einfo);
		}
*/
		codeinfo* codeInfo;
		bool success = analyzeMethod(meth, &codeInfo, &einfo);
		tidyAnalyzeMethod(&codeInfo);
		if (success == false) {
			throwError(&einfo);
		}
	}

	/* Allocate stack space and locals. */
	lcl = alloca(sizeof(struct _slots) * (meth->localsz + meth->stacksz));

	mobj = 0;
	npc = 0;

	/* If we have any exception handlers we must prepare to catch them.
	 * We also need to catch if we are synchronised (so we can release it).
	 */
	vmExcept_setIntrpFrame(&mjbuf, 0, meth, mobj);
	if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
		mjbuf.prev = (VmExceptHandler*)unhand(tid)->exceptPtr;
		unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
	}

	if (meth->exception_table != 0) {
		if (JTHREAD_SETJMP(mjbuf.jbuf) != 0) {
			meth = vmeth;
			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
			npc = vmExcept_getPC(&mjbuf);
			sp = &lcl[meth->localsz];
			sp->v.taddr = (void*)unhand(tid)->exceptObj;
			unhand(tid)->exceptObj = 0;
			engine(meth, lcl, sp, npc, retval, &mjbuf, tid, NULL);
			goto end;
		}
	}

	/* Calculate number of arguments */
	idx = sizeofSigMethod(meth, false);
	if (idx == -1) {
		throwError(&einfo);
	}
	idx += (methaccflags & ACC_STATIC ? 0 : 1);

	/* Copy in the arguments */
	sp = lcl;
	for (low = 0; low < idx; low++) {
		*(sp++) = *(arg++);
	}

	/* Sync. if required */
	if (methaccflags & ACC_SYNCHRONISED) { 
		if (methaccflags & ACC_STATIC) {
			mobj = &meth->class->head;
		}
		else {
			mobj = (Hjava_lang_Object*)lcl[0].v.taddr;
		}
		/* this lock is safe for Thread.stop() */
		lockObject(mobj);
		/* We must store the object on which we synchronized in
		 * the mjbuf chain or else the exception handler routine
		 * won't find it.
		 */
		vmExcept_setSyncobj(&mjbuf, mobj);
	}       

	sp = &lcl[meth->localsz - 1];

	engine(meth, lcl, sp, npc, retval, &mjbuf, tid, NULL);

 end:
	/* Unsync. if required */
	if (mobj != 0) {
		unlockObject(mobj);
	}       
	if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
		unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)mjbuf.prev;
	}

RDBG(	dprintf("Returning from method %s%s.\n", meth->name->data, METHOD_SIGD(meth)); )
}




/**
 * Evaluates method METH, local variables beginns at LCL and the
 * operand stack at SP0.  NEW_PC is used to implement exception
 * handling, i.e., is only non-zero during exception handling.  If the
 * method retuns a value it is stored into retval.  The current PC is
 * stored in mjbuf (before each instruction).  TID is used to
 * implement synchronization.
 *
 * If LABELS is non-NULL a pointer to an array containing the machine
 * code labels is stored into LABELS and the function returns
 * immediately (and all other arguments are ignored).
 **/
static void engine(methods* meth, slots *  lcl, 
		   slots * sp0, uintp  new_pc, 
		   slots * retval, volatile VmExceptHandler* mjbuf,
		   Hjava_lang_Thread  *tid,
		   Instr **labels) {
	Instr *__ip=NULL;	/* instuction pointer */
	Instr *icodep;		/* the first instruction of this method. */
	slots *sp = sp0;	/* stack pointer */
	jlong lcc;
	jlong tmpl;
	slots tmp[1];
	slots mtable[1];
	void check_label(Instr label) { label_2_string(label); }
	void dump_stack(void) {
		slots *p;					
		dprintf("----------- top");			
		for(p=MAX(sp,lcl); p >= MIN(sp0,lcl)-1; p--) {	
			if (p==lcl+meth->localsz-1)		
			    dprintf("\n----------- locals");	
			dprintf("\n[%#Lx]",p[0].v.tlong);	
			if (p==sp) dprintf(" <-- sp");		
			if (p==lcl) dprintf(" <-- fp");		
								
		}						
		dprintf("\n----------- bottom \n");		
	}

	/* Variables used directly in the machine */
	int32 idx;
	jint low;
	jint high;

	/* Call, field and creation information */
	callInfo cinfo;
	fieldInfo finfo;
	Hjava_lang_Class* crinfo;
	errorInfo einfo;

	if (labels) {		
		static Instr local_labels[256+N_CUSTOM_INSTRUCTIONS+1] = {
#       	  define instr(name,opcode,label,len,syn,body) {&&##label},
#       	  define undef(name,opcode,label,len,syn,body) {&&undef},
#       	  define custm(name,opcode,label,len,syn,body) {&&##label},
#       	  include "threaded_int.def"
#       	  undef instr
#       	  undef undef
#       	  undef custm
		  {&&undef}
		};
		*labels = local_labels; return;

	}

	if(((Instr *)meth->icode)[0].p==&&translate){translate_to_icode(meth);}
	icodep=meth->icode;
	JMP(icodep+new_pc);
#	define undef(name, opcode, label, length, syntax, body) 
#       define instr(name, opcode, label, length, syntax, body)  \
        label:							 \
             IDBG( dprintf("%03d: %s\n", PC, #label);)           \
             body						 \
             NEXT(length);	
#       define custm instr	
#       include "threaded_int.def"
#       undef instr
#       undef undef
#       undef custm
 undef: 
	fprintf(stderr,"undefined opcode.\n"); 
	assert(0);
 end:
	return;
}



static Instr *b2i = NULL;

/* get_translate_label -- code label which triggers translation. */
Instr  get_translate_label(void) {
	return get_labels()[TRANSLATE];
}

/* put_labels -- store addresses of intermediate instructions in LABELS. */
Instr* get_labels() { 
	if (b2i==NULL) engine(0,0,0,0,0,0,0,&b2i);
	return b2i;
}

/* make linker happy */
char* getEngine() {return "kaffe.threaded_int";}
char* engine_name = "Threaded Code";
char* engine_version = KVER;
