//
// de.gyronet.kangaroo.io.FileDescriptor.c
// kangaroo
//
// Created by Marc Kleine-Budde on 03.12.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from: java.io.FileInputStream.c
//			      java.io.FileOutputStream.c
// from the Kaffe Project
//
// see original Copyright below... 

/*
 * java.io.FileInputStream.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */
/*
 * java.io.FileOutputStream.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "config-io.h"
#include "files.h"
#include "defs.h"
#include "jsyscall.h"
#include "../../../kaffe/kaffevm/support.h"
#include "../../../kaffe/kaffevm/stringSupport.h"
#include "javax_microedition_io_Connector.h"
#include "de_gyronet_kangaroo_io_FileDescriptor.h"


void de_gyronet_kangaroo_io_FileDescriptor_open0
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this,
 struct Hjava_lang_String* name, jint mode)
{
	char str[MAXPATHLEN];
	int fd;
	int rc;
	int flags = 0;

	
	stringJava2CBuf(name, str, sizeof(str));

	switch (mode) {
	case javax_microedition_io_Connector_READ:
		flags = O_BINARY|O_RDONLY;
		break;
	case javax_microedition_io_Connector_WRITE:
		// FIXME: kangaroo: make difference between append and truncate mode

		flags = O_BINARY|O_WRONLY|O_CREAT|O_TRUNC;
		//flags = O_BINARY|O_WRONLY|O_CREAT|O_APPEND
		break;
	case javax_microedition_io_Connector_READ_WRITE:
		flags = O_BINARY|O_RDWR|O_CREAT;
		break;
	}

	rc = KOPEN(str, flags, 0, &fd);
	if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}

	unhand(this)->fd = fd;
}


void de_gyronet_kangaroo_io_FileDescriptor_close0
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this)
{
	int r;

	if (unhand(this)->fd >= 0) {
		r = KCLOSE(unhand(this)->fd);
		unhand(this)->fd = -1;
		if (r) {
			SignalError("java.io.IOException", SYS_ERROR(r));
		}
	}
}


jint de_gyronet_kangaroo_io_FileDescriptor_readBytes
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this,
 HArrayOfByte* buf, jint off, jint len)
{
	int rc;
	ssize_t ret;

	rc = KREAD(unhand(this)->fd, &unhand_array(buf)->body[off], len, &ret);
	if (rc == EINTR) {
		SignalError("java.io.InterruptedIOException", "");
	} else if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}
	return (ret > 0 ? ret : -1);
}


jint de_gyronet_kangaroo_io_FileDescriptor_read
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this)
{
	ssize_t ret;
	int rc;
	unsigned char byte;

	rc = KREAD(unhand(this)->fd, &byte, 1, &ret);
	if (rc == EINTR) {
		SignalError("java.io.InterruptedIOException", "");
	} else if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}
	return (ret > 0 ? byte : -1);
}

void de_gyronet_kangaroo_io_FileDescriptor_write
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this, jint byte)
{
	int fd;
	int r;
	unsigned char b = byte;
	ssize_t bwritten;

	fd = unhand(this)->fd;
	r = KWRITE(fd, &b, 1, &bwritten);
	if (r) {
		SignalError("java.io.IOException", SYS_ERROR(r));
	}
}

void de_gyronet_kangaroo_io_FileDescriptor_writeBytes
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this,
 HArrayOfByte* buf, jint off, jint len)
{
	int r, fd;
	ssize_t nw;

	if (off < 0 || off + len > obj_length(buf)) {
		SignalError("java.lang.IndexOutOfBoundsException", "");
	}
	fd = unhand(this)->fd;
	while (len > 0) {
		r = KWRITE(fd, &unhand_array(buf)->body[off], len, &nw);
		if (r != 0) {
			SignalError("java.io.IOException", SYS_ERROR(r));
		}
		off += nw;
		len -= nw;
	}
}

jint de_gyronet_kangaroo_io_FileDescriptor_available0
(struct Hde_gyronet_kangaroo_io_FileDescriptor* this)
{
	int r, nr;
	int fd = unhand(this)->fd;
	off_t cur = 0;

	r = KLSEEK(fd, cur, SEEK_CUR, &cur);
	if (r == 0) {
		struct stat statbuf;
		if ((KFSTAT(fd, &statbuf) == 0) && S_ISREG(statbuf.st_mode)) {
			return (statbuf.st_size - cur);
		}
	}

	/* If lseek or fstat fail, try another mechanism... */

#if defined(HAVE_IOCTL) && defined(FIONREAD)
	/* XXX make part of jsyscall interface */
	r = ioctl(fd, FIONREAD, &nr);
	if (r >= 0 && nr != 0) {
		return (nr);
	} else
		/* FIONREAD may report 0 for files for which data is
                   available; maybe select will do... */
#endif
	{
		/* This uses select() to work out if we can read - but
		 * what happens at the end of file?  */
		static struct timeval tm = { 0, 0 };
		fd_set rd;

		FD_ZERO(&rd);
		FD_SET(fd, &rd);
		KSELECT(fd+1, &rd, NULL, NULL, &tm, &r);
		if (r == 1) {
			nr = 1;
		}
		else {
			nr = 0;
		}

		return (nr);
	}
}
