//
// de.gyronet.kangaroo.io.InetImpl.c
// kangaroo
//
// Created by Marc Kleine-Budde on 04.04.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from: java.net.InetAddress.c
//			      java.net.PlainSocketImpl.c
// from the Kaffe Project
//
// see original Copyright below... 

/*
 * java.net.InetAddress.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

/*
 * java.net.PlainSocketImpl.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "config-net.h"
#include "config-io.h"
#include "../../../kaffe/kaffevm/gtypes.h"
#include "../../../kaffe/kaffevm/object.h"
#include <native.h>
#include "../../../kaffe/kaffevm/itypes.h"
#include "../../../kaffe/kaffevm/stringSupport.h"
#include "../../../kaffe/kaffevm/support.h"
#include "nets.h"
#include "jsyscall.h"
#include "../../../kaffe/kaffevm/debug.h"
#include "javax_microedition_io_SocketConnection.h"
#include "de_gyronet_kangaroo_io_InetImpl.h"

#ifdef KAFFE_VMDEBUG
static char *
ip2str(jint addr) 
{
	static char addrbuf[16];

	sprintf(addrbuf, "%u.%u.%u.%u",
	  (addr >> 24) & 0xff,
	  (addr >> 16) & 0xff,
	  (addr >>  8) & 0xff,
	  (addr      ) & 0xff);
	return addrbuf;
}
#endif /* KAFFE_VMDEBUG */

/*
 * Supported socket options
 */
  static const struct {
	  int jopt;
	  int level;
	  int copt;
  } socketOptions[] = {
#ifdef SO_SNDBUF
    { javax_microedition_io_SocketConnection_SNDBUF,		SOL_SOCKET,	SO_SNDBUF },
#endif
#ifdef SO_RCVBUF
    { javax_microedition_io_SocketConnection_RCVBUF,		SOL_SOCKET,	SO_RCVBUF },
#endif
#ifdef SO_LINGER
    { javax_microedition_io_SocketConnection_LINGER,		SOL_SOCKET,	SO_LINGER },
#endif
#ifdef TCP_NODELAY
    { javax_microedition_io_SocketConnection_DELAY,		IPPROTO_TCP,	TCP_NODELAY },
#endif
  };

#ifdef KAFFE_VMDEBUG
/*
 * Option names (for debugging only)
 */
  static const struct {
	  int opt;
	  char *name;
  } optionNames[] = {
#ifdef SO_SNDBUF
    { javax_microedition_io_SocketConnection_SNDBUF, "SO_SNDBUF" },
#endif
#ifdef SO_RCVBUF
    { javax_microedition_io_SocketConnection_RCVBUF, "SO_RCVBUF" },
#endif
#ifdef SO_LINGER
    { javax_microedition_io_SocketConnection_LINGER, "SO_LINGER" },
#endif
#ifdef TCP_NODELAY
    { javax_microedition_io_SocketConnection_DELAY, "TCP_NODELAY" },
#endif
  };
#endif /* KAFFE_VMDEBUG */


/*
 * Close this socket.
 */
void
de_gyronet_kangaroo_io_InetImpl_close0
(struct Hde_gyronet_kangaroo_io_InetImpl* this)
{
	int rc;

	DBG(NATIVENET,
	    dprintf("socketClose(%p)\n", this);
	    )

	if (unhand(this)->fd != -1) {
		rc = KSOCKCLOSE(unhand(this)->fd);
		unhand(this)->fd = -1;
		if (rc) {
			SignalError("java.io.IOException", SYS_ERROR(rc));
		}
	}
}

jint
de_gyronet_kangaroo_io_InetImpl_readBytes
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 HArrayOfByte* buf, jint offset, jint len)
{
        ssize_t r;
	int rc;
	int fd;

	DBG(NATIVENET,
	    dprintf("socket_read(%p, %p, %d, %d)\n", 
		    this, buf, offset, len);
	    )

	fd = unhand(this)->fd;
	if (fd < 0) {
		SignalError("java.io.IOException", "fd invalid"); 
	}

        rc = KSOCKREAD(fd, &unhand_array(buf)->body[offset], len, unhand(this)->timeout, &r);
	if (rc == EINTR) {
		SignalError("java.io.InterruptedIOException", 
			    "Read timed out or was interrupted");
	}
        if (rc) {
                SignalError("java.io.IOException", SYS_ERROR(rc));
        } else if (r == 0 && len > 0) {
                return (-1);    /* EOF */
        } else {
                return (r);
        }
}

void
de_gyronet_kangaroo_io_InetImpl_writeBytes
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 HArrayOfByte* buf, jint offset, jint len)
{
	int rc;
	int fd;
	ssize_t nw;

	DBG(NATIVENET,
	    dprintf("socket_write(%p, %p, %d, %d)\n", 
		    this, buf, offset, len);
	    )

	fd = unhand(this)->fd;
	if (fd >= 0) {
		while (len > 0) {
			rc = KSOCKWRITE(fd,
			    &unhand_array(buf)->body[offset], len, &nw);
			if (rc) {
				SignalError("java.io.IOException", SYS_ERROR(rc));
			}
			offset += nw;
			len -= nw;
		}
	} else {
		SignalError("java.io.IOException", "fd invalid"); 
	}
}

/*
 * Return how many bytes can be read without blocking.
 */
jint
de_gyronet_kangaroo_io_InetImpl_available0
(struct Hde_gyronet_kangaroo_io_InetImpl* this)
{
	int rc;
	jint len;
	int fd;
#if (!(defined(HAVE_IOCTL) && defined(FIONREAD)) && !defined(__WIN32__))
	static struct timeval tm = { 0, 0 };
	fd_set rd;
#endif

	DBG(NATIVENET,
	    dprintf("socketAvailable(%p)\n", this);
	    )
	fd = unhand(this)->fd;

#if defined(HAVE_IOCTL) && defined(FIONREAD)
	/* XXX make part of system call interface to protect errno */
	rc = ioctl(fd, FIONREAD, &len);
	if (rc < 0) {
		SignalError("java.io.IOException", SYS_ERROR(errno));
	}
#else
#if defined(__WIN32__) /* Windows hack - XXX */
	len = 0;
#else
	/* This uses KSELECT() to work out if we can read - but what
	 * happens at the end of file?
	 */

	FD_ZERO(&rd);
	FD_SET(fd, &rd);
	KSELECT(fd+1, &rd, NULL, NULL, &tm, &rc);	/* XXX ignore ret code*/
	if (rc == 1) {
		len = 1;
	} else {
		len = 0;
	}
#endif	/* defined(__WIN32__) */
#endif  /* defined(HAVE_IOCTL) && defined(FIONREAD) */

	DBG(NATIVENET,
	    dprintf("socketAvailable(%p) -> %d\n", this, len);
	    )

	return (len);
}

/*
 * Create a stream or datagram socket.
 */
void
de_gyronet_kangaroo_io_InetImpl_create0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 jboolean stream)
{
	int fd;
	int type;
	int rc;

	if (stream == 0) {
		type = SOCK_DGRAM;
	} else {
		type = SOCK_STREAM;
	}

	DBG(NATIVENET,
	    dprintf("socketCreate(%p, %s)\n", this, stream ? "stream" : "datagram");
	    )

	rc = KSOCKET(AF_INET, type, 0, &fd);
	if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}

	DBG(NATIVENET,
	    dprintf("socketCreate(%p, %s) -> fd=%d\n", 
		    this, stream ? "stream" : "datagram", fd);
	    )

	unhand(this)->fd = fd;
}

/*
 * Connect the socket to someone.
 */
void
de_gyronet_kangaroo_io_InetImpl_connect0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 jint daddr, jint dport)
{
	int fd;
	int rc;
	struct sockaddr_in addr;
	int alen;

	DBG(NATIVENET,
	    dprintf("socketConnect(%p, %s, %d)\n", 
		    this, ip2str(daddr), dport);
	    )

	memset(&addr, 0, sizeof(addr));
#if defined(BSD44)
	addr.sin_len = sizeof(addr);
#endif
	addr.sin_family = AF_INET;
	addr.sin_port = htons(dport);
	addr.sin_addr.s_addr = htonl(daddr);

	fd = unhand(this)->fd;
	rc = KCONNECT(fd, (struct sockaddr*)&addr, sizeof(addr), unhand(this)->timeout);
	if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}

	/* Enter information into socket object */
	alen = sizeof(addr);
	rc = KGETSOCKNAME(fd, (struct sockaddr*)&addr, &alen);
	if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}

	DBG(NATIVENET,
	    dprintf("socketConnect(%p, %s, %d) -> (lport: %d)\n",
		    this, ip2str(daddr), dport,
		    ntohs(addr.sin_port)
		    );
	    )

	unhand(this)->localport = ntohs(addr.sin_port);
	unhand(this)->localaddr = ntohl(addr.sin_addr.s_addr);
	unhand(this)->port = dport;
	unhand(this)->addr = daddr;
}

jint
de_gyronet_kangaroo_io_InetImpl_getSocketOption0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 jbyte opt)
{
	int k, rc, v;
	int vsize = sizeof(v);

	DBG(NATIVENET,
	    char *optstr = "UNKNOWN";
	    for (k = 0; k < sizeof(optionNames) / sizeof(optionNames[0]); k++) 
		    if (optionNames[k].opt == opt)
			    optstr = optionNames[k].name;
	    dprintf("socketGetOption(%p, %s)\n", this, optstr);
	    )

	/* Do easy cases */
	for (k = 0; k < sizeof(socketOptions) / sizeof(*socketOptions); k++) {
		if (opt == socketOptions[k].jopt) {
			rc = KGETSOCKOPT(unhand(this)->fd,
				socketOptions[k].level, socketOptions[k].copt,
				&v, &vsize);
			if (rc) {
				SignalError("java.io.IOException", SYS_ERROR(rc));
			}
			DBG(NATIVENET,
			    dprintf("socketGetOption(%p, -) -> %d\n", this, v);
			    )
			return v;
		}
	}

	SignalError("java.io.IOException", "Unimplemented socket option");    
}

void
de_gyronet_kangaroo_io_InetImpl_setSocketOption0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 jbyte opt, jint value)
{
	int k, rc;

	DBG(NATIVENET,
	    char *optstr = "UNKNOWN";
	    for (k = 0; k < sizeof(optionNames) / sizeof(optionNames[0]); k++) 
		    if (optionNames[k].opt == opt)
			    optstr = optionNames[k].name;
	    dprintf("socketSetOption(%p, %s, arg=%d)\n", this, optstr, value);
	    )

	/* Do easy cases */
	for (k = 0; k < sizeof(socketOptions) / sizeof(*socketOptions); k++) {
		if (opt == socketOptions[k].jopt) {
			rc = KSETSOCKOPT(unhand(this)->fd,
				socketOptions[k].level, socketOptions[k].copt,
				&value, sizeof(value));
			if (rc) {
				SignalError("java.io.IOException", SYS_ERROR(rc));
			}
			return;
		}
	}

	SignalError("java.io.IOException", "Unimplemented socket option");    
}

void
de_gyronet_kangaroo_io_InetImpl_acceptAndOpen0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 struct Hde_gyronet_kangaroo_io_InetImpl* con)
{
	int r;
	int rc;
	int alen;
	struct sockaddr_in addr;

	memset(&addr, 0, sizeof(addr));
#if defined(BSD44)
	addr.sin_len = sizeof(addr);
#endif
	addr.sin_family = AF_INET;
	addr.sin_port = htons(unhand(con)->localport);
	addr.sin_addr.s_addr = htonl(unhand(con)->localaddr);

	DBG(NATIVENET,
	    dprintf("socketAccept(%p, localport=%d, addr=%s, timeout=%d)\n", 
		    this, ntohs(addr.sin_port), ip2str(ntohl(addr.sin_addr.s_addr)), unhand(this)->timeout);
	    )

	alen = sizeof(addr);
	rc = KACCEPT(unhand(this)->fd, (struct sockaddr*)&addr, &alen, unhand(this)->timeout, &r);
	if (rc == EINTR) {
		SignalError("java.io.InterruptedIOException", 
			    "Accept timed out or was interrupted");
	}
	if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}
	unhand(con)->fd = r;

	/* Enter information into socket object */
	alen = sizeof(addr);
	r = KGETPEERNAME(r, (struct sockaddr*)&addr, &alen);
	if (r) {
		SignalError("java.io.IOException", SYS_ERROR(r));
	}

	unhand(con)->addr = ntohl(addr.sin_addr.s_addr);
	unhand(con)->port = ntohs(addr.sin_port);

	DBG(NATIVENET,
	    dprintf("socketAccept(%p, localport=-, addr=-) -> (sock: %p; addr: %s; port:%d)\n", 
		    this, con, ip2str(ntohl(addr.sin_addr.s_addr)), ntohs(addr.sin_port));
	    )

}

void 
de_gyronet_kangaroo_io_InetImpl_bind0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 jint laddr, jint lport)
{
	int rc;
	struct sockaddr_in addr;
	int fd;
	int on = 1;
	int alen;

	/*
	 * nomally INADDR_ANY _is_ 0, set this just in case it is not
	 */
	if (laddr == 0) {
		laddr = INADDR_ANY;
	}

	DBG(NATIVENET,
	    dprintf("socketBind(%p, %s, %d)\n", 
		    this, ip2str(laddr), lport);
	    )

	memset(&addr, 0, sizeof(addr));
#if defined(BSD44)
	addr.sin_len = sizeof(addr);
#endif
	addr.sin_family = AF_INET;
	addr.sin_port = htons(lport);
	addr.sin_addr.s_addr = htonl(laddr);

	fd = unhand(this)->fd;

	/* Allow rebinding to socket - ignore errors */
	(void)KSETSOCKOPT(fd, SOL_SOCKET, SO_REUSEADDR, (char*)&on, sizeof(on));
	rc = KBIND(fd, (struct sockaddr*)&addr, sizeof(addr));
	switch( rc ) {
	case 0:
		break;
	case EADDRNOTAVAIL:
	case EADDRINUSE:
	case EACCES:
		SignalError("java.io.IOException", SYS_ERROR(rc));
		break;
	default:
		SignalError("java.io.IOException", SYS_ERROR(rc));
		break;
	}

	/* Enter information into socket object */
 	unhand(this)->localaddr = laddr;
	if (lport == 0) {
		alen = sizeof(addr);
		rc = KGETSOCKNAME(fd, (struct sockaddr*)&addr, &alen);
		if (rc) {
			SignalError("java.io.IOException", SYS_ERROR(rc));
		}
		lport = ntohs(addr.sin_port);
	}
	unhand(this)->localport = lport;
	unhand(this)->localaddr = laddr;

	DBG(NATIVENET,
	    dprintf("socketBind(%p, %s, -) -> (lport: %d)\n", this,
		    ip2str(laddr), lport);
	    );

}

void
de_gyronet_kangaroo_io_InetImpl_listen0
(struct Hde_gyronet_kangaroo_io_InetImpl* this,
 jint count)
{
	int rc;

	DBG(NATIVENET,
	    dprintf("socketListen(%p, count=%d)\n", this, count);
	    )

	rc = KLISTEN(unhand(this)->fd, count);
	if (rc) {
		SignalError("java.io.IOException", SYS_ERROR(rc));
	}

}
