#include <jni.h>
#include "i802.h"
#include "jsyscall.h"

#include <stdio.h>
#include <openssl/ssl.h>
#include <openssl/tls1.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <sys/errno.h>
#include <asm/errno.h>

#define NOTIMEOUT 0

#define DEBUG
#ifdef DEBUG
#define DBG(args...)	fprintf(stderr, ##args)
#else
#define DBG(...)
#endif

extern void i802_ssl_init(void);
extern void  Java_java_lang_Thread_yield(void);

/* Set up a socket to listen - returns -1 or socket descriptor */
/* listen(int port) */
JNIEXPORT
jint
Java_com_instant802_cldc_ssl_Socket_listen(JNIEnv *env, jobject ob, jint port) {
	int err, flags;
	int listen_sd;
	struct sockaddr_in sa_serv;

	i802_ssl_init();

	/* ----------------------------------------------- */
	/* Prepare TCP socket for receiving connections */

	err = KSOCKET (AF_INET, SOCK_STREAM, 0, &listen_sd);

	memset (&sa_serv, '\0', sizeof(sa_serv));
	sa_serv.sin_family      = AF_INET;
	sa_serv.sin_addr.s_addr = INADDR_ANY;
	sa_serv.sin_port        = htons (port);		/* Server Port number */

	if (0 ==  (err = KBIND(listen_sd, (struct sockaddr*) &sa_serv,
			       sizeof (sa_serv)))) {
		err = KLISTEN(listen_sd, 5);
	    }
	if (0 != err)
		listen_sd = -1;
	return (jint) listen_sd;
}

/** Blocking accept. Returns -1 if an error, 0 if no connection received, sd if connection accepted */
/* accept(sd) */
JNIEXPORT
jint
Java_com_instant802_cldc_ssl_Socket_accept(JNIEnv *env, jobject ob, jint listen_sd) {
	int sd;
	struct sockaddr_in sa_cli;
	size_t client_len;
	int err;

	i802_ssl_init();
	client_len = sizeof(sa_cli);
	err = KACCEPT (listen_sd, (struct sockaddr*) &sa_cli, &client_len,
		       NOTIMEOUT, &sd);
	DBG("Accept returned %d error %d\n", sd, err);
	if (err != 0) {
		sd = -1;
		DBG("Closing listening socket %d\n", listen_sd);
		KSOCKCLOSE (listen_sd);
	}
	return (jint)sd;
}

/* Returns when socket connects */
JNIEXPORT
jint
Java_com_instant802_cldc_ssl_Socket_connect(JNIEnv *env, jobject ob,
				jbyteArray jaddr, jint addrlen, jint port) {
	int flags;
	char *paddr;
	int err;
	int sd;
	struct sockaddr_in sa_serv;

	i802_ssl_init();

	paddr = malloc(addrlen + 1);

	(*env)->GetByteArrayRegion(env, jaddr, 0, addrlen, paddr);
	paddr[addrlen] = 0;
	/* ----------------------------------------------- */
	/* Prepare TCP socket for receiving connections */

	err = KSOCKET (AF_INET, SOCK_STREAM, 0, &sd);
	memset (&sa_serv, '\0', sizeof(sa_serv));
	sa_serv.sin_family      = AF_INET;
	sa_serv.sin_addr.s_addr = inet_addr(paddr);
	sa_serv.sin_port        = htons (port);	/* Server Port number */
	err = KCONNECT(sd, (struct sockaddr*) &sa_serv, sizeof (sa_serv),
		       NOTIMEOUT);
	DBG("Connect error code %d\n", err);
	if (0 != err) {
		sd = -1;
	}
	/* clean up */
	free(paddr);
	return (jint) sd;
}

JNIEXPORT
void
Java_com_instant802_cldc_ssl_Socket_close(JNIEnv *env, jobject ob, jint sd) {
	KSOCKCLOSE (sd);
}

JNIEXPORT
jint
Java_com_instant802_cldc_ssl_Socket_read(JNIEnv *env, jobject ob, jint fd, jbyteArray jbuf, jint len) {
	int val, err;
	unsigned char *pbuf;

	i802_ssl_init();
	pbuf = (unsigned char *) malloc(len);
	err = KSOCKREAD(fd, pbuf, len, NOTIMEOUT, &val);
	DBG("Read returned %d error %d\n", val, err);
	if (err == 0) {
		(*env)->SetByteArrayRegion(env, jbuf, 0, len, pbuf);
	} else {
		val = err;
	}
	free(pbuf);
	return (jint) val;
}

/**
 * Write.
 * @return > 0 => # bytes written, 0 => try again, < 0 => error
 * */
JNIEXPORT
jint
Java_com_instant802_cldc_ssl_Socket_write(JNIEnv *env, jobject ob,
	    jint fd, jbyteArray jbuf, jint len) {
	int val, err;
	unsigned char *pbuf;

	i802_ssl_init();
	pbuf = (unsigned char *) malloc(len);
	(*env)->GetByteArrayRegion(env, jbuf, 0, len, pbuf);
	err = KWRITE(fd, pbuf, len, &val);
	DBG("Write returned %d error %d\n", val, err);
	if  (err != 0) {
		val = -1;
	}
	free(pbuf);
	return (jint)val;
}
