/*
 * java.lang.String.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "config-std.h"
#include "config-std.h"
#include "../../../kaffe/kaffevm/stringSupport.h"
#include "java_lang_String.h"
#include  "../../../kaffe/kaffevm/classMethod.h"
#include  "../../../kaffe/kaffevm/baseClasses.h"
#include  "../../../kaffe/kaffevm/soft.h"

Hjava_lang_String*
 java_lang_String_intern0(Hjava_lang_String* str) 
{ 
 	Hjava_lang_String *ret = stringInternString(str); 

 	if (!ret) { 
 		errorInfo info; 
 		postOutOfMemory(&info); 
 		throwError(&info); 
 	} 
 	return ret; 
} 


jint
java_lang_String_hashCode(Hjava_lang_String* string) {
	jint hash;
	int k;

	if (unhand(string)->hash != 0) {
		return(unhand(string)->hash);
	}
	for (k = hash = 0; k < STRING_SIZE(string); k++) {
	       hash = (31 * hash) + STRING_DATA(string)[k];
	}
	unhand(string)->hash = hash;
	return(hash);
}

jboolean 
java_lang_String_equals(Hjava_lang_String *s1, Hjava_lang_Object *so2) {
	Hjava_lang_Class *class;
	Hjava_lang_String *s2;
	int i;
	
	if (s1 == s2)
		return true;

	class = OBJECT_CLASS(so2);
	if (! instanceof(StringClass, class))
		return false;

	s2 = (Hjava_lang_String *) so2;

	if (STRING_SIZE(s1) != STRING_SIZE(s2))
		return false;

	for (i = 0; i < STRING_SIZE(s1); ++i)
		if (STRING_DATA(s1)[i] != STRING_DATA(s2)[i])
			return false;
	return true;
}

jint
java_lang_String_indexOf(Hjava_lang_String* str, Hjava_lang_String* pat, jint offset)
{
  jchar *a;
  jchar *p;

  int   n;
  int   m;
  int   m2;
  jint  i, k;
  unsigned char  bs[256];
  int *ibs;

  if (pat == 0) {
    SignalError("java.lang.NullPointerException", "");
  }

  if ( !str ) return -1;

  a = &(unhand_array(unhand(str)->value)->body[unhand(str)->offset]);
  n = unhand(str)->count;

  p = &(unhand_array(unhand(pat)->value)->body[unhand(pat)->offset]);
  m = unhand(pat)->count;
  m2 = m * 2;

  if ( m > n ) return -1;

  if ( offset < 0 ) offset = 0;

  if ( (m < 3) || (n < 128) || (m > 256) ) {
	/*
	 * If the pattern is too small, iterating the searched array would be
	 * cheaper than to set up the badmatch table, or the pattern length exceeds
	 * what we can store in the badmatch table, we revert to plain old brute force
	 */
	k = n - m+1;
	for ( i=offset; i<k; i++ ) {
	  if ( memcmp( &a[i], p, m2) == 0 )
		return i;		
	}
  }
  else {
	/*
	 * Otherwise we use the Quick search variant of Boyer-Moore for 
	 * O(n/m) lookup. Note that the (char) cast in the badmatch table access
	 * is the justification for turning this into a native method
	 */

	/* set up the badmatch table */
	k = (m << 24) | (m<<16) | (m<<8) | m;
	for ( i=0, ibs = (int*)bs; i<64; i++ )
	  ibs[i] = k; /* the default jump: m */
	for ( i=0; i<m; i++ )
	  bs[ (unsigned char)p[i] ] = m-i;

	k= n - m+1;
	for ( i=offset; i <= k; ) {
	  if ( memcmp( &a[i], p, m2) == 0 )
		return i;
	  i += bs[ (unsigned char)a[i+m] ];
	}
  }

  return -1;
}
