//
// de.gyronet.kangaroo.io.ConnectionManager.java
// kangaroo
//
// Created by Volker Gies on Thu Jul 18 2002.
// Copyright (c) 2002, 2003 GYRO net GmbH. All rights reserved.
//

package de.gyronet.kangaroo.io;

import java.io.IOException;
import java.util.Hashtable;
import javax.microedition.io.Connection;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.io.Connector;

/**
 * @version $Id: ConnectionManager.java,v 1.1.1.1 2003/10/31 00:47:08 carter Exp $
 * @author Volker Gies
 *
 */
public class ConnectionManager extends Connector {
    protected static Hashtable t = new Hashtable();

    static {
	t.put("console", "de.gyronet.kangaroo.io.ConsoleConnection");
	t.put("error", "de.gyronet.kangaroo.io.ErrorConnection");
	t.put("file", "de.gyronet.kangaroo.io.FileConnection");
	t.put("jar", "de.gyronet.kangaroo.io.JarConnection");
	t.put("resource", "de.gyronet.kangaroo.io.ResourceConnection");
	t.put("socket", "de.gyronet.kangaroo.io.TCPFactory");
    }
 
    public Connection openConnection(String name, int mode, boolean timeouts)
	throws IOException, ConnectionNotFoundException, IllegalArgumentException {

	String scheme = null;
	String target = null;
	Hashtable params = new Hashtable();
	
	int index;
  
	index = name.indexOf(':');
	if (index < 1) {
	    throw new IllegalArgumentException("could not parse scheme");
	}

	scheme = name.substring(0, index);
	name = name.substring(index + 1);
	index = name.indexOf(';');

	if(index < 1) {
	    target = name;
	} else {
	    target = name.substring(0, index);
	    name = name.substring(index + 1);
	    
	    do {
		String key;
		String value;
		
		index = name.indexOf('=');
		if(index < 1) {
		    throw new IllegalArgumentException("could no parse parameters");
		}

		key = name.substring(0, index);
		name = name.substring(index + 1);
		index = name.indexOf(';');
		
		if(index < 1) {
		    value = name;
		} else {
		    value = name.substring(0, index);
		    name = name.substring(index + 1);
		}
		
		params.put(key, value);
	    } while(index > 0);
	}

	String clazz = (String)(t.get(scheme));
	if(clazz == null) {
	    throw new ConnectionNotFoundException("unknown scheme");
	}

	try {
	    Protocol p = (Protocol)(Class.forName(clazz).newInstance());
	    return(p.open(target, params, mode, timeouts));
	} catch(Exception e) {
	    throw new ConnectionNotFoundException(e.getMessage());
	}
    }

    public static void register(String scheme, String clazz) throws IOException {
	if (t.containsKey(scheme)) {
	    throw new IOException("scheme already registered");
	}
	t.put(scheme, clazz);
    }

    public static void unregister(String scheme) throws IOException {
	t.remove(scheme);
    }
}
