//
// java.lang.Thread.java
// kangaroo
//
// Created by Marc Kleine-Budde on 02.10.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from: java.lang.Thread.java
//			      java.lang.ThreadGroup.java
// from the Kaffe Project
//
// see original Copyright below... 

/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */


package java.lang;

public class Thread implements Runnable {

    private int         priority = NORM_PRIORITY;
    private Thread 	threadQ;
    private Runnable	target;

    private char[] name;
    private static int	nthreads = 1;
    private static int	threadCount;

    private boolean	started;
    private boolean	dying;

    private Object	sleeper;
    private Object	holder;

    private boolean	daemon;

    private ClassLoader	context;
    private int		needOnStack;
    private Throwable	death;
    private Throwable	exceptObj;
    private Throwable	outOfMemoryError;
    private Throwable	stackOverflowError;

    private kaffe.util.Ptr PrivateInfo;
    private kaffe.util.Ptr exceptPtr;
    private kaffe.util.Ptr sem;
    private kaffe.util.Ptr sArg;
    private kaffe.util.Ptr sFunc;
    private Thread nextlk;
    
    public final static int MIN_PRIORITY = 1;
    public final static int NORM_PRIORITY = 5;
    public final static int MAX_PRIORITY = 10;

    private static class Sleeper {
    }

    private static class Suspender {
    }

    public Thread() {
        this(null);
    }

    public Thread(Runnable target) {
        Thread parent = currentThread();

        this.priority = parent.getPriority();
	this.stackOverflowError = new StackOverflowError();
	this.outOfMemoryError = new OutOfMemoryError();
        this.target = target;
	this.name = generateName().toCharArray();

        setPriority(priority);
	nthreads++;
    }

    public static int activeCount() {
	return nthreads;
    }

    native public static Thread currentThread();

    public void destroy() {
	synchronized(this) {
	    dying = true;
	    notifyAll();     // in case somebody is joining on us
	}

	nthreads--;

	destroy0();
    }

    native private void destroy0();

    public final int getPriority() {
	return priority;
    }

    public final boolean isAlive () {
	return (started && !dying);
    }
    
    public final void join() throws InterruptedException {
    	join(0);
    }

    public final synchronized void join(long millis) throws InterruptedException
	{
    	join(millis, 0);
    }

    public final synchronized void join(long millis, int nanos) throws InterruptedException
	{
    	Thread curr = currentThread();

    	/* Wait while time remains and alive */
    	if (millis == 0 && nanos == 0) {
    		while (isAlive()) {
    			curr.waitOn(this, 0);
    		}
    	}
    	else {
    		long end = System.currentTimeMillis() + millis;
    		for (;;) {
    			long remain = end - System.currentTimeMillis();
    			if (!isAlive() || remain <= 0) {
    				break;
    			}
    			curr.waitOn(this, remain);
    		}
    	}
    }
    public void run() {
	if (target != null) {
	    target.run();
	}
    }

    public final void setPriority(int newPriority) {
	if (newPriority < MIN_PRIORITY || newPriority > MAX_PRIORITY) {
	    throw new IllegalArgumentException("Priority: " + newPriority);
	}
	setPriority0(newPriority);
    }

    native private void setPriority0(int prio);

    public static void sleep(long millis) throws InterruptedException {
	Thread curr = Thread.currentThread();

	if (curr.sleeper == null) {
	    curr.sleeper = new Sleeper();
	}
	/* If we're sleeping for 0 time, then we'll really sleep for a very
	 * short time instead.
	 */
	if (millis == 0) {
	    millis = 1;
	}
	synchronized(curr.sleeper) {
	    curr.waitOn(curr.sleeper, millis);
	}	
    }

    public synchronized void start() {
	if (dying) {
	    throw new IllegalThreadStateException("Dead Thread");
	}
	if (started) {
	    throw new IllegalThreadStateException("Active Thread");
	}
	started = true;
	start0();
    }

    native public void start0();

    public String toString() {
	return new String("Thread["
			  + hashCode()
			  + ","	
			  + getPriority()
			  + "]"
			  );
    }

    public static void yield() {
	yield0();
    }

    native public static void yield0();

    void waitOn(Object hold, long timeout) throws InterruptedException {
	if (timeout < 0) {
	    throw new IllegalArgumentException("timeout is negative");
	}
// 	if (interrupting) {
// 	    interrupting = false;
// 	    throw new InterruptedException();
// 	}
	holder = hold;
	hold.wait0(timeout);
	holder = null;
// 	if (interrupting) {
// 	    interrupting = false;
// 	    throw new InterruptedException();
// 	}
    }

	/*
	 * This isn't infact a good idea since the Thread class isn't final and we can't
	 * guarantee this is ever called - which seems like dumb semantics to me but there
	 * you go.
	 */
	protected void finalize() throws Throwable {
		finalize0();
		super.finalize();
	}
	
	final native private void finalize0();

	
    /*
     * Called by system when thread terminates (for whatever reason)
     */
    /* Not private for now so GCJ won't optimize it away */
    void finish() {
	synchronized(this) {
	    dying = true;
	    notifyAll();     // in case somebody is joining on us
	}

	nthreads--;
    }

    private static String generateName() {
	return new String("Thread-" + threadCount++);
    }
}
