#ifndef __LCL_OPERATIONS_H__
#define __LCL_OPERATIONS_H__

#include<stdarg.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "xmlutils.h"
#include "operations.h"

/* columns present in station config tables */
#define OPER_MAC_ADDRESS                "mac"
#define OPER_RTS_THRESHOLD              "rts-threshold"
#define OPER_SHORT_RETRY_LIMIT          "dot11-short-retry-limit"
#define OPER_LONG_RETRY_LIMIT           "dot11-long-retry-limit"
#define OPER_FRAGMENTATION_THRESHOLD    "fragmentation-threshold"
#define OPER_MAX_TRANSMIT_MSDU_LIFETIME "dot11-max-transmit-msdu-lifetime"
#define OPER_MAX_RECEIVE_LIFETIME       "dot11-max-receive-lifetime"
#define OPER_MANUFACTURER_ID            "manufacturer-id"
#define OPER_PRODUCT_ID                 "manufacturer-product-name"

struct oper_data_info {
	char *mac_address;
	int mac_address_len;
	char *rts_threshold;
	int rts_threshold_len;
	char *short_retry_limit;
	int short_retry_limit_len;
	char *long_retry_limit;
	int long_retry_limit_len;
	char *fragmentation_threshold;
	int fragmentation_threshold_len;
	char *max_trans_msdu_lifetime;
	int max_trans_msdu_lifetime_len;
	char *max_recv_lifetime;
	int max_recv_lifetime_len;
	char *manufacturer_id;
	int manufacturer_id_len;
	char *product_id;
	int product_id_len;
	char *bss_value;
	int bss_value_len;
	char *radio_value;
	int radio_value_len;
};

static int CheckAndRetrieveMacAddress(xmlDocPtr s_doc);
static int CheckAndRetrieveRTSThreshold(xmlDocPtr s_doc);
static int CheckAndRetrieveShortRetryLimit(xmlDocPtr s_doc);
static int CheckAndRetrieveLongRetryLimit(xmlDocPtr s_doc);
static int CheckAndRetrieveFragmantationThreshold(xmlDocPtr s_doc);
static int CheckAndRetrieveMaxTransmitMSDU(xmlDocPtr s_doc);
static int CheckAndRetrieveMaxRecvLifetime(xmlDocPtr s_doc);
static int CheckAndRetrieveManufacturerID(xmlDocPtr s_doc);
static int CheckAndRetrieveProductID(xmlDocPtr s_doc);
static int CheckAndRetrieveBssValueForOperInfo(xmlDocPtr d_doc);
static int CheckAndRetrieveRadioValueForOperInfo(xmlDocPtr d_doc);

static void lcl_mac_address_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_rts_threshold_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_short_retry_limit_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_long_retry_limit_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_fragmentation_threshold(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_manufacturer_id_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_max_transmit_msdu_lifetime_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_product_id_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_max_recv_lifetime_xml(xmlDocPtr oper_info_doc, xmlNodePtr cur);
static void lcl_bss_for_oper_info_xml(xmlDocPtr sta_doc, xmlNodePtr cur);
static void lcl_radio_for_oper_info_xml(xmlDocPtr sta_doc, xmlNodePtr cur);

static int GetOperationMacAddress(xmlDocPtr, int);
static void GetRTSThreshold(xmlDocPtr, int);
static void GetShortRetryLimit(xmlDocPtr, int);
static void GetLongRetryLimit(xmlDocPtr, int);
static void GetFragmentationThreshold(xmlDocPtr, int);
static void GetMaxTransmitMSDULifetime(xmlDocPtr, int);	
static void GetMaxReceiveLifetime(xmlDocPtr, int);	
static void GetManufacturerID(xmlDocPtr, int);	
static void GetProductID(xmlDocPtr, int);
#endif  /* __LCL_OPERATIONS_H__*/
