/**
 * i802 DOM oriented methods.
 * Copyright 2004 Instant802 Networks, Inc. All Rights Reserved.
 */

/**
 * Return a boolean value telling whether
 * the first argument is an Array object.
 */
function isArray() {
	if (typeof arguments[0] == 'object') {
	  var criterion = arguments[0].constructor.toString().match(/array/i);
	   return (criterion != null);
	}
	return false;
}



function visitChildren( object, action ) {
	visitChildrenOfTagType( object, null, action );
}

/**
 * Used to apply a defined action on all children of the given object.
 * The children may be filtered via the tags array parameter ( e.g. { "TD", "A" } .)
 * The action paramter defines waht occurs on each visited child.  The return value of the
 * action can be null or a function that reverts the effects of the action function.
 */
function visitChildrenOfTagType( object, tags, action ) {
	if(object.childNodes) {
		var children = object.childNodes;
		for(var i = 0; i < children.length; i++ ) {
			var child = children[i];
			if( ( child.tagName && child.tagName.length > 0 ) && ( !tags || isTag( child, tags ) ) ) { // null indcates all tags.
				var revert = action( child );
				if( revert != null ) { // check if action should be revertable
					if(!child.reverts) child.reverts = new Array();
					child.reverts[action.actionId] = revert;  // invoke action and push revert.
				}
			}
			visitChildrenOfTagType( child, tags, action );
		}
	}
}

/**
 * Reverts the last set of actions taken for every child node of the given object.
 * The actionIds parameter may be a string or array of strings.  A null actionId
 * implies all last actions.
 */
function revertChildActions( object, actionIds ) {
	if(object.childNodes) {
		var children = object.childNodes;
		for(var i = 0; i < children.length; i++ ) {
			var child = children[i];
			if(child.reverts) {
				if(actionIds) {
					if(isArray(actionIds)) {
						for(var i = 0; i < actionIds.length; i++) {
							if(child.reverts[actionIds[i]]) {
								child.reverts[actionIds[i]]( child );
								child.reverts[actionIds[i]] = null;
							}
						}
					}
					else {
						if(child.reverts[actionIds]) {
							child.reverts[actionIds]( child );
							child.reverts[actionIds] = null;
						}
					}
				}
				else {
					if(child.reverts.length > 0) {
							child.reverts[child.reverts.length-1]( child );
							child.reverts.length--;
					}
				}
			}
			revertChildActions( child, actionIds );
		}
	}
}

/**
 * Looks for any child node with .
 */
function enableChildren( object ) {
	revertChildActions( object, "i8_disable" );
}

/**
 * Looks for any child node with an onclick and disallows user mouse input along with a visual indication.
 * TBD: expand to form objects and links.
 */
function disableChildren( object ) {
	var returnFalse = function() { return false };
	
	var actionId = "i8_disable";
	
	var action = function( o ) { 
		if(!o.i8_disable) { // save the original enabled color and onclick properties.
			o.i8_disable = new Object();
			if(o.style) {
				o.i8_disable.color = o.style.color ? o.style.color : "";
				o.style.color = "#808080";
			}
			if(o.onclick) {
				o.i8_disable.onclick = o.onclick; o.onclick = returnFalse;
			}
		}
		var revert = function( o ) {
			if(!o.i8_disable) return;
			if(o.i8_disable.color || o.i8_disable.color == "") {
				o.style.color = o.i8_disable.color;
			}
			if(o.i8_disable.onclick) {
				o.onclick = o.i8_disable.onclick;
			}
			o.i8_disable = null;
		};
		return revert;
	};
	action.actionId = "i8_disable";
	
	visitChildren( object, action );	
}
/**
 * Looks for any child node with .
 */
function unhideChildren( object ) {
	revertChildActions( object, "i8_hide" );
}

/**
 * Looks for any child node with an onclick and disallows user mouse input along with a visual indication.
 * TBD: expand to form objects.
 */
function hideChildren( object ) {
	var actionId = "i8_hide";
	
	var action = function( o ) { 
		if(!o.i8_hide) { // save the original display property value.
			o.i8_hide = new Object();
			if(o.style) {
				o.i8_hide.display = o.style.display ? o.style.display : "";
				o.style.display = "none";
			}
		}
		var revert = function( o ) {
			if(!o.i8_hide) return;
			if(o.i8_hide.display || o.i8_hide.display == "") {
				o.style.display = o.i8_hide.display;
			}
			o.i8_hide = null;
		};
		return revert;
	};
	action.actionId = "i8_hide";
	
	visitChildren( object, action );	
}

/**
 * Utility function to determine if a given node is a tag.
 */
function isTag( object, tagNames ) {
	if(isArray(tagNames)) {
		for(var i = 0; i < tagNames.length; i++) {
			if(tagNames[i] == object.tagName) return true;
		}
		return false;
	}
	else {
		return (object.tagName == tagNames);
	}
}

/**
 * Force a recalulation of the layout of the contents.
 */
function layoutContent() {
	postLayoutContent();
}

function postLayoutContent() {
 	window.setTimeout('_layoutContent()', 0);
} 

function _layoutContent() {
  	var content = document.getElementById("content");
	
	if(content) {
		window.status += "*" + content.style.width;
    	if(content.style.height == "100%") {
			content.style.height = "99%";
		} else {
			content.style.height = "100%";
		}
		window.status += ":" + content.style.width + "," + content.style.display + "]";
	}
	else {
		alert("layoutContent(), unable to recalulate layout.  content id not found.");
	}
 }

