/**
 * apc_web_handler.h - Declarations for the web handler.
 * Copyright: Copyright 2004 Instant802 Networks, Inc. All Rights Reserved.
 */

#ifndef APC_WEB_HANDLER_DOT_H
#define APC_WEB_HANDLER_DOT_H

#include "autoconf.h"

#include "apc_web_types.h"
#include <lsqlquery/value.h>
#include <lsqlquery/connector.h>

#define APC_WEB_NON_TEMPLATE_HANDLER 1
#define APC_WEB_TEMPLATE_FIELD_INPUT_HANDLER 2
#define APC_WEB_TEMPLATE_FIELD_OUTPUT_HANDLER 3

#define APC_WEB_IF_LAN "br0"

#ifdef CONFIG_OAP_WAN_INTERFACE
#define APC_WEB_IF_WAN "eth0"
#endif

#ifdef CONFIG_OAP_GUEST
#define APC_WEB_IF_GUEST "brguest"
#endif

#define APC_WEB_R1 "wlan0"

#ifdef CONFIG_OAP_GUEST
#define APC_WEB_R1B1 "wlan0bssInternal"
#else
#define APC_WEB_R1B1 "wlan0bss"
#endif

#define APC_WEB_R1B1S1 "wlan0"

#ifdef CONFIG_OAP_GUEST
#define APC_WEB_R1B2 "wlan0bssGuest"
#define APC_WEB_R1B2S1 "wlan0guest"
#endif

#ifdef CONFIG_OAP_RADIOS_2

#define APC_WEB_R2 "wlan1"

#ifdef CONFIG_OAP_GUEST
#define APC_WEB_R2B1 "wlan1bssInternal"
#else
#define APC_WEB_R2B1 "wlan1bss"
#endif

#define APC_WEB_R2B1S1 "wlan1"

#ifdef CONFIG_OAP_GUEST
#define APC_WEB_R2B2 "wlan1bssGuest"
#define APC_WEB_R2B2S1 "wlan1guest"
#endif

#endif /* CONFIG_OAP_RADIOS_2 */


#ifdef CONFIG_OAP_VIRTUAL_WIRELESS_NETWORKS

#define APC_WEB_VWN1BR "brvwn1"

#define APC_WEB_R1VWN1 "wlan0bssvwn1"
#define APC_WEB_R1VWN1S "wlan0vwn1"

#define APC_WEB_VWN2BR "brvwn2"

#define APC_WEB_R1VWN2 "wlan0bssvwn2"
#define APC_WEB_R1VWN2S "wlan0vwn2"

#ifdef CONFIG_OAP_RADIOS_2
#define APC_WEB_R2VWN1 "wlan1bssvwn1"
#define APC_WEB_R2VWN1S "wlan1vwn1"

#define APC_WEB_R2VWN2 "wlan1bssvwn2"
#define APC_WEB_R2VWN2S "wlan1vwn2"
#endif /* CONFIG_OAP_RADIOS_2 */

#endif /* CONFIG_OAP_VIRTUAL_WIRELESS_NETWORKS */

#define APC_WEB_IF_PPP "ppp0"

/**
 * Setup the link list of non template handlers.
 * Args: the apc_web_interface_t, and one of *_HANDLER
 * Returns: none
 * Side-effects: none
 */
void apc_web_setup_handlers(apc_web_interface_t *interface, int handler_type);

/**
 * Gets the handler for the given page or field.
 * Args: apc_web_interface_t, one of APC_WEB_*_HANDLER, page
 * Returns: an apc_web_handler_t
 * Side-effects: none
 */
apc_web_handler_t *apc_web_get_handler(apc_web_interface_t *inteface,
					int handler_type, char *key,
					int key_len);

/**
 * Adds a handler to the linked list.
 * Args: the prev entry, the key of the handler, the apc_web_handler_t
 * Returns: nothing.
 * Side-effects: *cur->next will be set to something.
 */
void apc_web_add_handler(apc_web_handler_table_entry_t **cur, char *key,
			apc_web_handler_t *handler);

/**
 * Creates a new template_field_output-handler.
 * Args: fn ptr to the template_field_output_handler.
 * Returns: a new apc_web_handler_t
 * Side-effects: none.
 */
apc_web_handler_t *new_template_field_output_handler(template_field_output_handler fn);

/**
 * Creates a new template_field_input-handler.
 * Args: fn ptr to the template_field_input_handler.
 * Returns: a new apc_web_handler_t
 * Side-effects: none.
 */
apc_web_handler_t *new_template_field_input_handler(template_field_input_handler fn);

#ifdef CONFIG_OAP_GUEST
int is_guest_enabled(void);
int is_guest_connection_type_ethernet(void);
#endif
char *get_ethernet_interface_name(const char *role);
#ifdef CONFIG_OAP_VLAN
char *get_vlan_name(const char *role);
#endif

#endif /* APC_WEB_HANDLER_DOT_H */
