/**
 * apc_web_template.h - Declarations for the web template engine.
 * Copyright: Copyright 2004 Instant802 Networks, Inc. All Rights Reserved.
 */

#ifndef APC_WEB_TEMPLATE_DOT_H
#define APC_WEB_TEMPLATE_DOT_H

#include <sys/types.h>
#include <stdio.h>

#include "apc_web_types.h"

/* commands have the following format 
 * <?i802 command(macro [, value]) ?>
 */
#define APC_WEB_TEMPLATE_COMMAND_SUB_STRING "sub"
#define APC_WEB_TEMPLATE_COMMAND_SELECT_STRING "selected"
#define APC_WEB_TEMPLATE_COMMAND_CHECK_STRING "checked"
#define APC_WEB_TEMPLATE_COMMAND_SUB 1
#define APC_WEB_TEMPLATE_COMMAND_SELECT 2
#define APC_WEB_TEMPLATE_COMMAND_CHECK 3

#define APC_WEB_TEMPLATE_COMMAND_PREFIX "<?i802 "
#define APC_WEB_TEMPLATE_COMMAND_SUFFIX " ?>"

typedef struct {
	char *prefix;		/* ptr to whatever matches
					APC_WEB_TEMPLATE_COMMAND_PREFIX */
	int prefix_len;
	char *command;		/* the specific command after the prefix */
	int command_num;	/* one of APC_WEB_TEMPLATE_COMMAND_* */
	char *macro;		/* the first arg to the command */
	int macro_len;
	char *candidate_value;	/* the second arg to the cmd, if it exists */
	int candidate_value_len;
	char *suffix;		/* ptr to whatever matches
					APC_WEB_TEMPLATE_COMMAND_SUFFIX */
	int suffix_len;
	int full_len;		/* full length of entire command */
} apc_web_template_command_t;

typedef struct {
	apc_web_interface_t *interface;	/* web interface globals */
	struct iovec *iov;	/* iovector for output */
	size_t iovlen;
	char *buffer;		/* buffer to hold the input */
	int buffer_len;
	char *buffer_position;	/* ptr to current position in the buffer */
	char *filename;		/* like it says */
	FILE *output;		/* where the output goes */
} apc_web_template_t;


/**
 * Processes an i802 template.
 * Args: dir of the template, filename of template, fd of output.
 * Returns: Non-zero on error. 
 * Side-effects: Prints processed template to standard out.
 */
int apc_web_process_template(apc_web_interface_t *interface,
				char * template_dir, char *file,
				FILE *output_handle); //, xmlNodePtr cur)

#endif
