/**
 * apc_web_types.h - Declarations for the web handler.
 * Copyright: Copyright 2004 Instant802 Networks, Inc. All Rights Reserved.
 */

#ifndef APC_WEB_TYPES_DOT_H
#define APC_WEB_TYPES_DOT_H

#define __USE_GNU 1
#include <string.h>

/* non-template handlers */
typedef void (*non_template_handler) (void);

/* field input handlers */

/**
 * template_field_input_handler - Handles an input macro.
 * @macro: the name of the macro.
 *         Used to get the value of the macro from the CGI library.
 * Returns: non-zero on error.
 * Notes: The macro arg here is a bit strange. after all, each
 *        each input handler should know internally which macro it
 *        responds to. It's merely here for ease of maintenance. We
 *        already have the macros mapped to input_handlers in
 *        apc_web_handler_input.c. so rather than hardcoding the macro
 *        in the input_handler, just trust that we've been assigned to
 *        respond to the appropriate macro. and when we need to change
 *        the macro, it's one fewer change to worry about.
 *        Also, it could potentially let us use the same handler
 *        for multiple macros.
 */
typedef int (*template_field_input_handler) (char *macro);

/* An input_record associates a macro name with a function. */
struct input_record {
	char *macro;
	template_field_input_handler function;
};
	
/* field output handlers */
typedef char *(*template_field_output_handler) (void);

/**
 * struct output_record - An output record for the template engine.
 * @macro:    the macro name.
 * @function: the function to call to get the value of @macro.
 */
struct output_record {
	char *macro;
	template_field_output_handler function;
};

typedef struct {
	int type;	/* one of *_HANDLER */
	union {
		non_template_handler non_template;
		template_field_input_handler input;
		template_field_output_handler output;
	} fn;
} apc_web_handler_t;

/* foreach type of handler there is a linked list
 * keyed by the macro name.
 */
typedef struct apc_web_handler_table_entry apc_web_handler_table_entry_t;
struct apc_web_handler_table_entry {
	char *key;
	apc_web_handler_t *handler;
	struct apc_web_handler_table_entry *next;
};

/* stuff needed by the web interface across requests */
typedef struct {
	apc_web_handler_table_entry_t *non_template_handler;
	apc_web_handler_table_entry_t *input_handler;
	apc_web_handler_table_entry_t *output_handler;
} apc_web_interface_t;

#endif
