/*
 * Created on Aug 13, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.Activation;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.instant802.cldc.DataPath.GenericRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.clustermastering.ClusterMaster;
import com.instant802.cldc.clustermastering.ClusterMasterAgent2;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.ControlledQueue;
import com.instant802.cldc.util.ControlledThread;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class Activator
{

	private static final String TYPE = "Activator";
	public static final int SERVICE_PORT = 39852;
	private ControlledQueue incomingConnections = new ControlledQueue();
	private final Hashtable services = new Hashtable();
	private ControlledThread receiver, processor;
	private final Vector instanceManagers = new Vector();
	private final OrchestratorNode node;

	public Activator()
	{
		if (Log.DEBUG) Log.warning(
			TYPE,
			"creating new activator with no OrchestratorNode - only useful for unit testing");
		node = null;
	}

	public Activator(OrchestratorNode node)
	{
		this.node = node;
	}

	/**
	 * @param string
	 */
	private void debug(String string)
	{
		Log.debug(TYPE, string);
	}

	Service startService(final String name) throws ServiceException
	{
		Enumeration e = instanceManagers.elements();
		while (e.hasMoreElements())
		{
			ServiceInstanceManager m = (ServiceInstanceManager) e.nextElement();
			if (m.hasType(name))
			{
				try
				{
					return m.getInstance(name, node, this);
				} catch (Exception e1)
				{
					throw new ServiceException(e1.getMessage());
				}
			}
		}
		throw new ServiceException("couldn't start service: " + name);
	}

	public void addInstanceManager(ServiceInstanceManager manager)
	{
		instanceManagers.addElement(manager);
	}

	public Service addService(final String name, final Service service)
	{
		services.put(name, service);
		return service;
	}

	/**
	 * return a service given a name.
	 * 
	 * @param name
	 *                  the name of the service
	 * @return the service
	 * @throws ServiceException
	 *                   thrown if an appropriate service couldn't be found
	 */
	public Service service(String name) throws ServiceException
	{
		Service found = (Service) services.get(name);
		if (null == found)
		{
			/* if the service can't be found, then try to start one */
			found = startService(name);
		}
		return found;
	}

	/**
	 * @param message
	 * @return
	 */
	InvocationResult invoke(final InvocationMessage message)
	{
		if (Log.DEBUG) debug("invoke called with message: " + message);
		/*
		 * TODO: this may be the place to handle errors and augment the result
		 * with an attribute to indicate the error.
		 */
		try
		{
			Service service = service(message.getService());
			return service.invokeMethod(message);
		} catch (NullPointerException e)
		{
			warning(e);
			return new InvocationResult(e);
		} catch (Exception e)
		{
			warning(e);
			return new InvocationResult(e);
		}
	}

	private void warning(Exception e)
	{
		Log.warning(TYPE, e);
	}

	public final void start() throws SocketException
	{
		receiver =
			new ConnectionReceiver(incomingConnections, node);
		processor = new ConnectionProcessor(this, incomingConnections);
		processor.start();
		receiver.start();
		if (Log.DEBUG) Log.notice("service activator started");
	}

	/**
	 *  
	 */
	public final synchronized void waitForStop() throws ActivatorException
	{
		incomingConnections.close();
		if (null != processor)
		{
			processor.waitForStop();
			processor = null;
		}
		if (null != receiver)
		{
			receiver.waitForStop();
			receiver = null;
		}
	}

	public boolean invokeServerMethod(
		String name,
		InvocationMessage message,
		InvocationResult result)
		throws IOException, SocketException, XMLRepresentationException
	{
		ActivatorClient client = new ActivatorClient();
		InvocationResult remoteresult;
		Enumeration records;
		GenericRecord record;
		ClusterMasterAgent2 cma =
			(ClusterMasterAgent2) node.clusterMasterAgent();
		ClusterMaster cm;

		if (null != cma)
		{
			cm = cma.currentMaster();
			if (null != cm)
			{
				client.setNodeAddress(cm.getAddress());
				remoteresult = client.InvokeMethod(message);
				records = remoteresult.records().elements();
				while (records.hasMoreElements())
				{
					record = (GenericRecord) records.nextElement();				
					result.add(record);
				}
				debug("Activator result: " + new String(result.asByteArray()));
				return true;
			}
		}
		return false;
	}
}
