/*
 * Created on Aug 13, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.channelplanner;

import java.util.Enumeration;

import com.instant802.cldc.Activation.Activator;
import com.instant802.cldc.Activation.ExceptionRecord;
import com.instant802.cldc.Activation.InvocationMessage;
import com.instant802.cldc.Activation.InvocationResult;
import com.instant802.cldc.Activation.Service;
import com.instant802.cldc.Activation.ServiceInstanceManager;
import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.deviceconfiguration.ClusteredAP;
import com.instant802.cldc.deviceconfiguration.ConfigObject;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.san.AP;
import com.instant802.cldc.san.APSanGraph;
import com.instant802.cldc.san.GraphInitFailureException;
import com.instant802.cldc.san.SAN;
import com.instant802.cldc.san.SanComputationException;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ChannelPlanner extends Service implements Runnable
{

	public static final String TYPE = "channel-planner";
	private final OrchestratorNode node;
	private boolean running;
	public final static String DISABLED_MODE = "none",
		PLAN_ONLY_MODE = "plan",
		PLAN_AND_EXECUTE_MODE = "execute";
	private ChannelOptimizationPlan current_plan;
	private long last_plan_ts;
	private ChannelPlanRecord cpr;
	private int state;

	public ChannelPlanner(OrchestratorNode node)
	{
		this.node = node;
		this.current_plan = null;
		this.state = ChannelPlanExecutor.PREPLAN;
		this.node.configurationModel().setChannelPlanner(this);
	}

	private void debug(String message)
	{
		if (Log.DEBUG)
			Log.debug(TYPE, message);
	}

	public void run()
	{
		ChannelPlanExecutor executor;
		last_plan_ts = 0L;
		// Here and not in ( )because the config model is build by starting the
		// node, not by constructing it (!)
		try
		{
			handleConfigChanged(node.configurationModel().getChannelPlannerConfig());
		}
		catch (Exception e)
		{
			Log.emergency("No/faulty channel plan configuration -- not running planner ..");
			return;
		}
		running = true;
		if (Log.DEBUG)
			debug("RunningChannel Planner");
		while (running)
		{
			if (node.coordinationAgent().isServer()
				&& ((System.currentTimeMillis() - last_plan_ts > cpr.getPlanInterval())
					|| 0L == last_plan_ts))
			{
				try
				{
					current_plan = generatePlan();
					if (null != current_plan)
					{
						state = current_plan.getState();
						if (!(current_plan instanceof DefaultChannelOptimizationPlan))
						{
							if (cpr.getMode().equals(PLAN_AND_EXECUTE_MODE))
							{
								executor =
									new ChannelPlanExecutor(
										cpr.getEnergyThreshold(),
										cpr.getWLANThreshold(),
										this.node);
								if (ChannelPlanExecutor.SUCCESS
									== (state = executor.execute(current_plan)))
								{
									current_plan.display();
									last_plan_ts = System.currentTimeMillis();
								}
								else
								{
									if (Log.DEBUG)
										debug(
											"Plan not executed - failed to meet constraints ["
												+ state
												+ "] pre="
												+ (long) current_plan.preenergy
												+ " post="
												+ (long) current_plan.postenergy);
								}
							}
						}
						else if (Log.DEBUG)
							debug("Default plan generated [" + state + "]");
					} 
					else 
					{
						// else we must be coordinating and planning is temporarily off
						// this is an obscure corner condition
						state = ChannelPlanExecutor.FAIL_OTHER;
					}
				}
				catch (Throwable e)
				{
					e.printStackTrace();
				}
			}
			try
			{
				Thread.sleep(60 * 1000L); // Wake up once a
				// minute to check
			}
			catch (Exception _)
			{
			}
		}
	}

	private ChannelOptimizationPlan generatePlan()
	{
		APSanGraph apsg;
		AP ap;
		float pre_energy, post_energy;
		ChannelOptimizationPlan plans = null;
		SAN san;
		WirelessNeighborhoodRecords records;

		records = WirelessNeighborhoodRecords.getWirelessNeighborhoodRecords();
		if (null == records)
		{ // Auto gather is off ?? -- we must be coordinating
			return null; // Since we don't have an APSG we can't produce a default
							// plan even
		}
		if (Log.DEBUG)
			debug("Planning ...");
		try
		{
			if (!cpr.getMode().equals(DISABLED_MODE))
			{
				
				apsg = new APSanGraph(records.getAPs(), cpr.getLockedAPs(), cpr.getChannelBag());	
				apsg.init();
				records.populateGraph(apsg);				
				san = new SAN(0.97f, 10000, 100.0f, 0.001f, 1);
				pre_energy = apsg.energy();
				san.run(apsg);
				post_energy = apsg.energy();
				plans = new ChannelOptimizationPlan(pre_energy, post_energy);
				// This ts is for a succesful plan only
				for (int i = 0; i < apsg.size(); ++i)
				{
					ap = apsg.getAP(i);
					/*
					 * We want to gather up all APs which are in our cluster - whether or
					 * not they are actually manageable - but they should be in our
					 * planning band
					 */
					if ((apsg.isManageable(ap) || apsg.isLocked(ap))
						&& ap.getBand().equals(apsg.currentBand()))
					{
						plans.addPlan(
							new Plan(
								ap.getIPAddress(),
								ap.getMACAddress(),
								ap.getBand(),
								ap.getWlanName(),
								ap.getInitialChannel(),
								ap.getChannel(),
								ap.load(),
								apsg.isLocked(ap)));
					}
				}
				plans.setState(ChannelPlanExecutor.SUCCESS);
			}
			else
			{
				plans = defaultChannelPlan();
				plans.setState(ChannelPlanExecutor.NOT_RUNNING);
			}
		}
		catch (GraphInitFailureException gi)
		{
			// Thrown if we have no manageable APs
			plans = defaultChannelPlan();
			plans.setState(ChannelPlanExecutor.FAIL_ALL_APS_LOCKED);
		}
		catch (SanComputationException sce)
		{
			if (Log.DEBUG)
				debug("SANcomputation error: " + sce.toString());
			plans = defaultChannelPlan();
			plans.setState(ChannelPlanExecutor.FAIL_OTHER);
		}
		catch (Exception e)
		{
			if (Log.DEBUG)
				debug("Error: " + e.toString());
			plans = defaultChannelPlan();
			plans.setState(ChannelPlanExecutor.FAIL_OTHER);
		}
		return plans;
	}

	/**
	 * The UI always requires a complete channel plan so it can show locked/unlocked,
	 * current channels etc. So we create a default channel plan for those boundary
	 * conditions under shich a CP would not have been created - internal error, all APs
	 * locked etc. @authornull plan *will* be returned in the case where the CP isn't yet
	 * initialized
	 */
	private DefaultChannelOptimizationPlan defaultChannelPlan()
	{
		DefaultChannelOptimizationPlan plans;
		AP ap, aps[];
		WirelessNeighborhoodRecords records;
		
		if (Log.DEBUG)
			debug("Creating default no-op plan");
		plans = new DefaultChannelOptimizationPlan(0.0f, 0.0f);		
		records = WirelessNeighborhoodRecords.getWirelessNeighborhoodRecords();
		if (null == records)
		{ // Auto gather is off ?? -- we must be coordinating
			return plans; 
		}
		aps = records.getAPs();	

		for (int i = 0; i < aps.length; ++i)
		{
			ap = aps[i];
			/*
			 * We want to gather up all APs which are in our cluster - whether or not
			 * they are actually manageable. Only add APs which are in our current band --
			 * reduce confusion in dual radio case
			 */
			if ((null != ap.getIPAddress()) && ap.getBand().equals(cpr.currentBand()))
			{
				plans
					.addPlan(new Plan(
						ap.getIPAddress(),
						ap.getMACAddress(),
						ap.getBand(),
						ap.getWlanName(),
						ap.getInitialChannel(),
						ap.getInitialChannel(),
						0L,
						// Same channel - change nothing plan
						cpr.getLockedAPs().contains(ap.getIPAddress())));
			}
		}
		return plans;
	}

	public void stop()
	{
		running = false;
	}

	public void handleConfigChanged(ConfigObject cp) throws XMLRepresentationException
	{
		String smode, svar;

		if (null == cp)
		{
			if (Log.DEBUG)
				debug("handleConfigChanged - null ConfigObject");
			return;
		}

		cpr = new ChannelPlanRecord(cp.asXMLDocument().getBytes());

		// Now enable or disable radio channel synchronization based on
		// whether we are planning
		if (cpr.getMode().equals(DISABLED_MODE))
			node.schema().addSharedField("radio", "static-channel");
		else
			node.schema().removeSharedField("radio", "static-channel");
		if (Log.DEBUG)
			debug("Channel Planner: mode is " + cpr.getMode());

		try
		{
			if (Log.DEBUG)
				debug(
					"Plan threshold energy="
						+ cpr.getEnergyThreshold()
						+ ", wlan="
						+ cpr.getWLANThreshold()
						+ ", plan="
						+ cpr.getPlanInterval());
		}
		catch (Exception e)
		{
			if (Log.DEBUG)
				debug("Parse error in channel_planner configuration: " + e.toString());
		}
	}

	static class InstanceManager extends ServiceInstanceManager
	{
		protected String typeName()
		{
			return ChannelPlanner.TYPE;
		}

		public Service getInstance(String name, OrchestratorNode node, Activator a)
			throws Exception
		{
			ChannelPlanner planner = new ChannelPlanner(node);
			a.addService(name, planner);
			new Thread(planner).start();
			return planner;
		}
	}

	protected boolean doMethod(String name, InvocationMessage message, InvocationResult result)
		throws Exception
	{
		if (!node.coordinationAgent().isServer())
		{
			return node.activator().invokeServerMethod(name, message, result);
		}
		else
			return doLocalMethod(name, message, result);
	}

	protected boolean doLocalMethod(
		String name,
		InvocationMessage message,
		InvocationResult result)
		throws Exception
	{
		if (null == cpr)
		{
			handleConfigChanged(node.configurationModel().getChannelPlannerConfig());
			if (null == cpr)
			{
				result.add(
					new ExceptionRecord(new Exception("No channel plan configuration - aborting")));
				return true;
			}
		}
		if (name.equals("current_plan"))
		{
			long ts;
			if (last_plan_ts == 0)
			{
				ts = -1;
			}
			else
			{
				ts = System.currentTimeMillis() - last_plan_ts;
			}
			cpr.field(ChannelPlanRecord.TS, "" + ts);
			cpr.field(ChannelPlanRecord.INTERFERENCE_RATIO, "" + getInterferenceRatio());
			cpr.field(ChannelPlanRecord.MSG, "" + state);
			result.add(cpr);
			if (null != current_plan)
			{
				Enumeration apsettings = current_plan.getPlans().elements();
				while (apsettings.hasMoreElements())
				{
					Plan applan = (Plan) apsettings.nextElement();
					APChannelPlanRecord apcp = new APChannelPlanRecord();
					apcp.field(APChannelPlanRecord.IP, applan.getIp());
					apcp.field(APChannelPlanRecord.BAND, applan.getBand());
					apcp.field(APChannelPlanRecord.PREV_CHANNEL, "" + applan.getOldchannel());
					apcp.field(APChannelPlanRecord.PLAN_CHANNEL, "" + applan.getNewchannel());
					apcp.field(APChannelPlanRecord.CUR_CHANNEL, "" + applan.getCurChannel(node));
					// Due to a wobbly UI conception (mine) while we would
					// naturally let the plan tell us
					// if the APis locked the user may have manually locked it
					// after the plan
					// was generated so we actually go tro the current
					// ChannlPlanRecord to tell us
					apcp.field(
						APChannelPlanRecord.LOCKED,
						cpr.isLocked(applan.getIp()) ? "true" : "false");
					result.add(apcp);
				}
			}
			return true;
		}
		else if (name.equals("execute"))
		{
			ChannelPlanExecutor executor =
				new ChannelPlanExecutor(
					cpr.getEnergyThreshold(),
					cpr.getWLANThreshold(),
					this.node);
			executor.forcedExecute(current_plan);
			return true;
		}
		else if (name.equals("update"))
		{
			String channelplanrecord = message.stringField("channel_plan_record");
			if (null != channelplanrecord)
			{
				cpr.updateRecord(new XMLRecord(channelplanrecord.getBytes()));
			}
			else
			{
				ExceptionRecord e =
					new ExceptionRecord(new Exception("bad ChannelPlan update - cpr"));
				result.add(e);
				return true;
			}
			cpr.saveState(node);
			return true;
		}
		else if (name.equals("neighborhood"))
		{
			WirelessNeighborhoodRecords records;
			records = WirelessNeighborhoodRecords.getWirelessNeighborhoodRecords();
		}
		else if (name.equals("start"))
		{
			cpr.setMode(PLAN_AND_EXECUTE_MODE);
			cpr.saveState(node);
			state = ChannelPlanExecutor.PREPLAN;
			return true;
		}
		else if (name.equals("stop"))
		{
			cpr.setMode(DISABLED_MODE);
			cpr.saveState(node);
			state = ChannelPlanExecutor.NOT_RUNNING;
			last_plan_ts = 0L; // Make sure to force plan on restart
			return true;
		}
		return false;
	}

	private int getInterferenceRatio()
	{
		if (null == current_plan)
			return 100;
		else
			return current_plan.preenergy == 0.0
				? 100
				: (int) (100 * (current_plan.postenergy / current_plan.preenergy));
	}
}

class DefaultChannelOptimizationPlan extends ChannelOptimizationPlan
{
	public DefaultChannelOptimizationPlan(float preenergy, float postenergy)
	{
		super(preenergy, postenergy);
	}
}
