/*
 * Created on Aug 13, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.channelplanner;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.instant802.cldc.Activation.Activator;
import com.instant802.cldc.Activation.ActivatorException;
import com.instant802.cldc.Activation.ExceptionRecord;
import com.instant802.cldc.Activation.InvocationMessage;
import com.instant802.cldc.Activation.InvocationResult;
import com.instant802.cldc.Activation.RecordCollection;
import com.instant802.cldc.Activation.Service;
import com.instant802.cldc.Activation.ServiceInstanceManager;
import com.instant802.cldc.DataPath.GenericRecord;
import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.LSQLClient;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.clustermembership.MembershipList;
import com.instant802.cldc.deviceconfiguration.ClusteredAP;
import com.instant802.cldc.hostapd.HostAPD;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.san.AP;
import com.instant802.cldc.util.Log;

/**
 * @author robin, alanl
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class LocalWirelessNeighborhoodMonitorImpl
	extends Service
	implements LocalWirelessNeighborhoodMonitor
{

	private final OrchestratorNode node;

	public LocalWirelessNeighborhoodMonitorImpl(OrchestratorNode node)
	{
		this.node = node;
	}

	private void debug(String message)
	{
		Log.debug("LocalWirelessNeighborhoodMonitorImpl", message);
	}

	public Enumeration getCurrentRecords()
	{
		return new WirelessNeighborhoodModel(node).currentRecords();
	}

	static class InstanceManager extends ServiceInstanceManager
	{
		protected String typeName()
		{
			return LocalWirelessNeighborhoodMonitor.TYPE;
		}
		
		public Service getInstance(
			String name,
			OrchestratorNode node,
			Activator a)
			throws Exception
		{
			LocalWirelessNeighborhoodMonitorImpl monitor =
				new LocalWirelessNeighborhoodMonitorImpl(node);
			a.addService(name, monitor);
			return monitor;
		}
	}

	// We must be able to re-enter to get channels, neighborhhods etc
	public boolean isThreadable()
	{
		return true;
	}

	protected boolean doMethod(
		String name,
		InvocationMessage message,
		InvocationResult result)
		throws Exception
	{
		try
		{
			if (name.equals("get_current_records"))
			{
				return mGetCurrentRecords(result);
			} else if (name.equals("setWANChannel"))
			{
				return mSetWANChannel(message);
			} else if (name.equals("getWANChannel"))
			{
				return mGetWANChannel(message, result);
			} else if (name.equals("get_clustered_wireless_neighborhood"))
			{
				if (!node.coordinationAgent().isServer())
				{
					return node.activator().invokeServerMethod(
						name,
						message,
						result);
				} else
					return mGetClusteredWirelessNeighborhood(message, result);
			} else if (
				name.equals("get_clustered_wireless_neighborhood_by_ap"))
			{
				if (!node.coordinationAgent().isServer())
				{
					return node.activator().invokeServerMethod(
						name,
						message,
						result);
				} else
					return mGetClusteredWirelessNeighborhoodByAP(
						message,
						result);
			}
		} catch (Exception e)
		{
			result.add(new ExceptionRecord(e));
			return true;
		} finally
		{
			System.gc();
		}
		return false;
	}

	/*
	 * To make life easier for Dave's UI which wants to display a nice array of
	 * [IP, MAC] = SIgnal we produce two structures in the result: <macs>
	 * <1:2:3:4:5:6> SS1,SS2,SS3,SS4,..SSn </1:2:3:4:5:6> ... </macs> Where
	 * Each MAC address corresponds to a unique MAC in teh neighborhood and the
	 * SSi are the signal strengths (normalized 0 - 70 on a log scale). <ips>
	 * <1.2.3.4> location </1.2.3.4> ... <IPn> .. </IPn></ips> IMPORTANT: SSi
	 * in MACj is the SS seen between MACj and the ith APas determined by the
	 * <ips> list. If it turns out there is no data between IPi and MACj we set
	 * SSI in the MACj row to -1. Further, we want the first n entries in the
	 * <macs> record to correspond to the APs in the <ips> record
	 */
	private boolean mGetClusteredWirelessNeighborhood(
		InvocationMessage message,
		InvocationResult result)
		throws ActivatorException
	{
		Enumeration records;
		NeighborAPRecord record;
		Hashtable aptallys = new Hashtable();
		Vector clustermacs = new Vector();
		Hashtable clusteredaps = new Hashtable();
		
		new MembershipList(node.masterCriteria(), node.configurationModel()).populate();

		records =
			WirelessNeighborhoodRecords
				.getWirelessNeighborhoodRecords()
				.getRecords();

		Enumeration ape = node.configurationModel().getClusteredAPs();

		while (ape.hasMoreElements())
		{
			ClusteredAP ap = (ClusteredAP) ape.nextElement();
			clusteredaps.put(ap.ipAddress().trim(), ap);
		}		
		while(records.hasMoreElements())
		{
			APTally aptally;
			record = (NeighborAPRecord) records.nextElement();
			// Check to see if it is really clustered
			if (clusteredaps.containsKey(record.getHostIP().trim()))
			{	
				if (null == (aptally = (APTally) aptallys.get(record.getHostIP())))
				{	
					aptally =
						new APTally(record.getLocation(), record.getHostMac(), record.getHostSSID());
					aptallys.put(record.getHostIP(), aptally);
					clustermacs.addElement(record.getHostMac());						
				}
				aptally.addNbrRecord(record);		
			} 
		}		
		MacsRecord macsrecord = new MacsRecord();
		IPsRecord ipsrecord = new IPsRecord();
		for (Enumeration ips = aptallys.keys(); ips.hasMoreElements(); )
		{
			String ip = (String) ips.nextElement();
			APTally tally = (APTally) aptallys.get(ip);
			ipsrecord.addIP(ip, tally.getLocation());
			macsrecord.addMac(aptallys,  tally.getRadioMac(),  tally.getSSID());
		}
		
		// Now go through all records looking for mac records whose radio mac is
		// not already in clustermacs
		for (Enumeration tallies = aptallys.elements(); tallies.hasMoreElements(); )
		{
			APTally tally = (APTally) tallies.nextElement();			
			for (Enumeration nbrrecords = tally.neighbors(); nbrrecords.hasMoreElements(); )
			{	
				String mac, ssid;
				record = (NeighborAPRecord) nbrrecords.nextElement();
				mac = record.getNbrMac();
				if (! clustermacs.contains(mac))
					macsrecord.addMac(aptallys,  mac, record.getSSID());
			}
		}
		result.add(macsrecord);
		result.add(ipsrecord);
		return true;
	}

	private boolean mGetClusteredWirelessNeighborhoodByAP(
		InvocationMessage message,
		InvocationResult result)
		throws ActivatorException
	{
		Enumeration records;
		NeighborAPRecord record;
		records =
			WirelessNeighborhoodRecords
				.getWirelessNeighborhoodRecords()
				.getRecords();	
		while (records.hasMoreElements())
		{
			record = (NeighborAPRecord) records.nextElement();
			if (message.stringField("ap") == null
				|| message.stringField("ap").equals(record.getHostIP()))
			{
				result.add(
					new NeighborhoodDetailRecord(
						record.getHostIP(),
						record.getNbrMac(),
						record.getSSID(),
						record.getSignal(),
						record.getChannel(),
						record.getBeaconInterval(),
						record.getLastBeaconTS(),
						record.getRate()));
			}
		}
		return true;
	}

	private boolean mGetWANChannel(
		InvocationMessage message,
		InvocationResult result)
	{
		String wlanname = message.stringField(ChannelPlanExecutor.WLAN_NAME);
		if (Log.DEBUG)
			debug("Getting " + wlanname + " channel ");
		ChannelRecord cr = new ChannelRecord();
		String channel = HostAPD.getHostAPD().getRadioChannel(wlanname);
		cr.field(ChannelPlanExecutor.CHANNEL_NUMBER, channel);
		if (Log.DEBUG)
			debug(wlanname + " channel = " + channel);
		result.add(cr);
		return true;
	}

	private boolean mSetWANChannel(InvocationMessage message)
		throws
			LSQLException,
			XMLRepresentationException,
			IOException,
			MalformedHTTPMessageException,
			SocketException
	{
		String wlanname = message.stringField(ChannelPlanExecutor.WLAN_NAME);
		int wlanchannel = message.intField(ChannelPlanExecutor.CHANNEL_NUMBER);

		if (Log.DEBUG)
			debug("Setting " + wlanname + " to channel " + wlanchannel);
		LSQLClient client = new LSQLClient(node.schema());
		client.query(
			"<update><radio name=\""
				+ wlanname
				+ "\"><static-channel>"
				+ wlanchannel
				+ "</static-channel></radio></update>");
		return true;
	}

	private boolean mGetCurrentRecords(InvocationResult result)
	{
		Enumeration e = getCurrentRecords();
		while (e.hasMoreElements())
		{
			result.add((GenericRecord) e.nextElement());
		}
		return true;
	}
}

// Local Classes

class APTally
{
	private String location, ssid;
	private Hashtable nbrs;
	private String radiomac;

	public APTally(String location, String mac, String ssid)
	{
		this.location = location;
		this.radiomac = mac;
		this.ssid = ssid;
		nbrs = new Hashtable();
	}

	public String getLocation()
	{
		return location;
	}

	public String getRadioMac()
	{
		return radiomac;
	}

	public int getSS(String macaddr)
	{
		NeighborAPRecord record = (NeighborAPRecord) nbrs.get(macaddr);
		if (null == record)
			return -1;
		else
			return record.getSignal() >= 0 ? record.getSignal() : 0;
	}
	
	public String getSSID() 
	{
		return ssid;
	}

	public void addNbrRecord(NeighborAPRecord record)
	{
		nbrs.put(record.getNbrMac(), record);
	}
	
	public Enumeration neighbors() 
	{
		return nbrs.elements();
	}
}

class ChannelRecord extends GenericRecord
{
	public String typeName()
	{
		return "ChannelRecord";
	}
}

class NeighborhoodDetailRecord extends GenericRecord
{
	public final static String HOST_IP = "ip";
	public final static String NEIGHBOR_MAC = "mac";
	public final static String SIGNAL_STRENGTH = "sig";
	public final static String CHANNEL = "chan";
	public final static String BEACON_INT = "b_int";
	public final static String BEACON_AGE = "b_age";
	public final static String RATE = "rate";
	public final static String SSID = "ssid";

	public NeighborhoodDetailRecord(
		String hostip,
		String radiomac,
		String ssid,
		int signal,
		int channel,
		int beaconint,
		long beaconage,
		int rate)
	{
		field(HOST_IP, hostip);
		field(NEIGHBOR_MAC, radiomac);
		field(SSID, ssid);
		field(SIGNAL_STRENGTH, "" + signal);
		field(CHANNEL, "" + channel);
		field(BEACON_INT, "" + beaconint);
		field(BEACON_AGE, "" + beaconage);
		field(RATE, "" + rate);
	}

	public String typeName()
	{
		return "NDR";
	}
}

class MacsRecord extends RecordCollection
{
	public String typeName()
	{
		return "macs";
	}

	/**
	 * Add mac record for ip to macs record --
	 */
	void addMac(Hashtable aptallys, String mac, String ssid)
	{		
		StringBuffer sss = new StringBuffer();
		// Always enumerate across keys to be sure to preserve key order
		for (Enumeration ips = aptallys.keys(); ips.hasMoreElements(); )
		{
			String ip = (String) ips.nextElement();
			APTally aptally = (APTally) aptallys.get(ip);
			if (0 != sss.length())
				sss.append(",");
			sss.append(aptally.getSS(mac));
		}
		addMac(mapmac(mac), sss.toString(), ssid);	
	}
	
	private void addMac(String mac, String record, String ssid)
	{
		try
		{
			add( new XMLRecord(("<" + mac + "><ss>" + record + "</ss><ssid>" +
											     ssid + "</ssid></" + mac + ">").getBytes()));
		} catch (XMLRepresentationException e)
		{
			e.printStackTrace();
		}
	}	
	
	private String mapmac(String macaddr)
	{
		return "_" + macaddr.replace(':', '_');
	}	
}

class IPsRecord extends RecordCollection
{
	public String typeName()
	{
		return "ips";
	}

	public void addIP(String ip, String record)
	{
		try
		{
			add(new XMLRecord(("<" + mapip(ip) + "><loc>" + record + "</loc></" + ip + ">").getBytes()));
		} catch (XMLRepresentationException e)
		{
			e.printStackTrace();
		}
	}
	
	private String mapip(String ip)
	{
		return "_" + ip.replace('.', '_');
	}	
}