/*
 * Created on Jun 21, 2004
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package com.instant802.cldc.channelplanner;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.instant802.cldc.Activation.ActivatorException;
import com.instant802.cldc.clustermembership.InvocationFailureException;
import com.instant802.cldc.deviceconfiguration.ClusteredAP;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.san.AP;
import com.instant802.cldc.san.APSanGraph;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class WirelessNeighborhoodRecords implements Runnable
{
	private LocalWirelessNeighborhoodMonitorFactory monitorFactory;
	private static final String TYPE = "WirelessNeighborhoodRecords";
	private Hashtable root, macIndex;
	private static WirelessNeighborhoodRecords wnr = null;
	private boolean rungather;
	private OrchestratorNode node;

	public WirelessNeighborhoodRecords(OrchestratorNode node)
		throws ActivatorException
	{
		monitorFactory = new LocalWirelessNeighborhoodMonitorFactory(node);
		root = new Hashtable(16);
		macIndex = new Hashtable();
		this.node = node;
	}
	
	public void startAutoGather() {
		if (Log.DEBUG) Log.debug("WirelessNeighborhoodRecords - starting AutoGather on server");
		rungather = true;
		if (null == wnr) {
			wnr = this;
			new Thread(this).start();
		}
	}
	
	public void stopAutoGather() {
		if (Log.DEBUG) Log.debug("WirelessNeighborhoodRecords - stoping AutoGather on server");		
		rungather = false;
		wnr = null;
	}

	public static WirelessNeighborhoodRecords getWirelessNeighborhoodRecords() {
		return wnr;
	}
	
	public synchronized void update()
	{
		Enumeration monitors = monitorFactory.getMonitors();
		Enumeration records;
		while (monitors.hasMoreElements())
		{
			LocalWirelessNeighborhoodMonitor monitor =
				(LocalWirelessNeighborhoodMonitor) monitors.nextElement();		
			try
			{
				records = monitor.getCurrentRecords();
				while (records.hasMoreElements())
				{
					WirelessNeighborhoodRecord record =
						(WirelessNeighborhoodRecord) records.nextElement();
					update(record);
				}
			} catch (InvocationFailureException e)
			{
				error(e);
			}
		}	
		prune();
	}

	// When new APs are detected in the cluster we probably want to force a 
	// neighborhood update rather than wait up to 15 minutes before we see them
    // (though their nbhd data will probably be sparse
	public static void updateNow() {
		if (null != wnr)
			wnr.update();
	}

	private void update(WirelessNeighborhoodRecord record)
	{
		HostRecord hostrecord;
		NeighborAPRecord ap;
		String host_mac = record.getHostRadioMac();
		
		if (null == (hostrecord = (HostRecord) root.get(host_mac)))
		{
			hostrecord = new HostRecord();
			root.put(host_mac, hostrecord);
		}
		hostrecord.update(record);
		if (null
			== (ap =
				(NeighborAPRecord) hostrecord.neighborhood().get(
					record.macAddress())))
		{
			ap = new NeighborAPRecord();
			hostrecord.neighborhood().put(record.macAddress(), ap);
		}
		ap.update(record);
	}

	private void prune()
	{
		Enumeration nbraprecords;
		NeighborAPRecord record;
		Vector toremove;

		toremove = new Vector();
		nbraprecords = getRecords();
		while (nbraprecords.hasMoreElements())
		{
			record = (NeighborAPRecord) nbraprecords.nextElement();
			if (!record.isFresh())
			{
				toremove.addElement(record);
				if (Log.DEBUG)
					debug("Pruning AP " + record.getNbrMac()+"/" + record.getHostMac());
			} 
		}
		nbraprecords = toremove.elements();
		while (nbraprecords.hasMoreElements()) {
			record = (NeighborAPRecord) nbraprecords.nextElement();
			getHostRecord(record.getHostMac()).neighborhood().remove(record.getNbrMac());
		}
	}

	public Enumeration getRecords()
	{
		return new Enumeration()
		{
			private Enumeration hostrecords = root.elements();
			private Enumeration nbrhoodrecords = null;
			private HostRecord hostrecord = null;

			public boolean hasMoreElements()
			{
				while (true)
				{
					if ((null != nbrhoodrecords)
						&& nbrhoodrecords.hasMoreElements())
						return true;
					if (hostrecords.hasMoreElements())
					{
						hostrecord = (HostRecord) hostrecords.nextElement();
						nbrhoodrecords = hostrecord.neighborhood().elements();
					} else
						return false;
				}
			}

			public Object nextElement()
			{
				return nbrhoodrecords.nextElement();
			}
		};
	}
	
	private HostRecord getHostRecord(String hostmac) {
		return (HostRecord) root.get(hostmac);
	}
	
	/**
	 * Generate list of APs - including hosts. Channels and IP addresses (for
	 * hosts) should be set. We address here the thorny issue of an AP that
	 * has been removed from a cluster. It still exists radio-wise so we cannot just drop
	 * it, and it may (indeed probably) is already in the root. Since SANGraph determines the
	 * manageability by (amongst other things) checking to make sure it has an IP address we
	 * check here to see if the IP address corresponds to an address still in the cluster before
	 * assign it to the AP object to be returned in the list.
	 * 
	 * @return
	 */
	public AP[] getAPs()
	{
		Enumeration hosts, ape;
		HostRecord hr;
		AP ap, aps[];

		hosts = root.keys();

		while (hosts.hasMoreElements())
		{
			String host_mac = (String) hosts.nextElement();
			hr = (HostRecord) root.get(host_mac);
			if (null == (ap = (AP) macIndex.get(host_mac)))
			{
				ap = new AP(host_mac, hr.channel(), hr.band());
				macIndex.put(host_mac, ap);
			}
			// 	The AP() may have been created because it appeared in a
			// neighborhood first
			// (i.e. it was seen wirelessly before being recognized as a host).
			// Also now check to make sure it is in the cluster
			ap.setIPAddress(null); // To make sure the cluster check gets it right !!!
			if (null != hr.ip()) 
			{	
				ape = node.configurationModel().getClusteredAPs();
				while (ape.hasMoreElements())
					if (((ClusteredAP) ape.nextElement()).ipAddress().equals(hr.ip())) {
						ap.setIPAddress(hr.ip());
						break;
					}
			}
			ap.setWlanName(hr.name());
			//The channel may have changed snce this AP() was created ...
			ap.setInitialChannel(hr.channel());
			// Now walk wireless neighborhodo for this host
			ape = hr.neighborhood().keys();
			while (ape.hasMoreElements())
			{
				String mac = (String) ape.nextElement();
				if (null == macIndex.get(mac))
				{
					NeighborAPRecord napr =
						(NeighborAPRecord) hr.neighborhood().get(mac);
					ap = new AP(mac, napr.getChannel(), napr.getHostBand());
					macIndex.put(mac, ap);
				}
			}
		}
		ape = macIndex.elements();
		aps = new AP[macIndex.size()];
		int index = 0;
		while (ape.hasMoreElements())
		{
			aps[index] = (AP) ape.nextElement();
			++index;
		}
		return aps;
	}

	private void debug(String message)
	{
		if (Log.DEBUG)
			Log.debug(TYPE, message);
	}

	private void error(Exception e)
	{
		Log.error(TYPE, e);
	}

	public void populateGraph(APSanGraph apsg)
	{
		Enumeration host_macs, wireless_macs;
		String host_mac, wireless_mac;
		long host_load;
		int signal;
		Hashtable neighborhood;

		host_macs = root.keys();
		while (host_macs.hasMoreElements())
		{
			host_mac = (String) host_macs.nextElement();
			neighborhood = ((HostRecord) root.get(host_mac)).neighborhood();
			wireless_macs = neighborhood.keys();
			while (wireless_macs.hasMoreElements())
			{
				wireless_mac = (String) wireless_macs.nextElement();
				NeighborAPRecord record =
					(NeighborAPRecord) neighborhood.get(wireless_mac);
				signal = record.getSignal();
				host_load = record.getHostLoad();
				try
				{
					if (macIndex.containsKey(host_mac))
					{
						if (macIndex.containsKey(wireless_mac))
						{
							AP ap1 = (AP) macIndex.get(host_mac);
							AP ap2 = (AP) macIndex.get(wireless_mac);
							apsg.setLink(
								ap1.getIndex(),
								ap2.getIndex(),
								signal);
							ap1.updateLoad(host_load);
						}
					}
				} catch (Exception e1)
				{
					if (Log.DEBUG)
						debug(
							"Error building APSanGraph model: "
								+ e1.toString());
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		while (rungather) {
			update();
			System.gc();
			try {
				Thread.sleep(15 * 60000L);
			} catch (InterruptedException e) {
				
			}
		}	
	}
}

class HostRecord
{
	/**
	 * HostRecord contains host info and table of neighboring APs. This must be
	 * updated each time through as ip, channel etc are dynamic and will change
	 * (e.g. channel changes on planning !). 
	 */
	private String host_ip;
	private int host_channel;
	private String host_name;
	private String host_band;
	private Hashtable neighborhood;

	public HostRecord()
	{
		neighborhood = new Hashtable(16);
	}

	public void update(WirelessNeighborhoodRecord record)
	{
		host_ip = record.getHostIP();
		host_name = record.getHostName();
		host_channel = record.getHostRadioChannel();
		host_band = record.getHostRadioBand();
	}

	public int channel()
	{
		return host_channel;
	}

	public String ip()
	{
		return host_ip;
	}

	public String name()
	{
		return host_name;
	}
	
	public String band() 
	{
		return host_band;
	}

	public Hashtable neighborhood()
	{
		return neighborhood;
	}
}
