/*
 * Created on Oct 28, 2003
 * author: Robin Barooah
 * (C) 2003 Instant802 Networks Inc
 */
package com.instant802.cldc.clustermastering;

import com.instant802.cldc.clustermembership.MembershipAgent;
import com.instant802.cldc.net.MacAddress;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 */
public class MasterCriteria {

	private static final long MEMBERS    = 0x0001000000000000l;
	private static final long PRECEDENCE = 0x0100000000000000l;
    private final OrchestratorNode node;
	private int ourMembers = 0;
	private String clusterId = MembershipAgent.NON_MEMBER;
	private boolean becomeMaster = false;
	private static long startTime = System.currentTimeMillis();
	private static int timeout = 10 * 60; // ten minute timeout for precedence to reach zero
	private static float precedenceScale = 253.0f / (float) timeout; 
		
	public synchronized long messageScore( AnnounceMasterMessage message ) {
		long score = 0;
		MacAddress mac = new MacAddress( message.getSenderId() );
		score += mac.asLong();
		score += (MEMBERS * message.members() );
		score += (PRECEDENCE * message.precedence() );
		return score;
	}

	public MasterCriteria(OrchestratorNode node) {
		this.node = node;
	}

	public synchronized void setMembers(int count) {
		ourMembers = count;
	}

	public synchronized void setClusterId(String id) {
		if (Log.DEBUG) debug("cluster id being set to "+id );
		clusterId = id;
	}

	private void debug(String string) {
		Log.debug("MasterCriteria", string );
	}

	public synchronized String getClusterID() {
		return clusterId;
	}

	public int members() {
		return ourMembers;
	}

	/**
	 * reset the criteria to those of a new ap.
	 */
	public void reset() {
		ourMembers = 0;
		clusterId = MembershipAgent.NON_MEMBER;
		becomeMaster = false;
	}
	
	/**
	 * sets the 'becomeMaster' flag.  When this is set
	 * the AP should try to become the master using the
	 * precedence rules in order to make a restored configuration
	 * become the configuration used by the whole cluster.
	 */
	public synchronized void setBecomeMaster() {
		this.becomeMaster = true;
	}
	
	private final int uptime() {
		return (int) ((System.currentTimeMillis() - startTime) / 1000 );
	}
	
	public int precedence() {
		if( ! becomeMaster )
			return 0;
		else 
		{
			return 1 +
				(int) (((float) ( timeout - uptime())) * precedenceScale) ;
		}
	}
}
