/*
 * Created on Nov 1, 2003
 * author: Robin Barooah
 * (C) 2003 Instant802 Networks Inc
 */
package com.instant802.cldc.deviceconfiguration;

import java.util.Enumeration;
import org.kxml2.kdom.Element;
import org.kxml2.kdom.Node;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 */
public class ConfigDOM extends DOMBased {

	private String DOCUMENT_TYPE = "I802config";
	final Element root;
	final Schema schema;

	public ConfigDOM( Schema schema, byte[] raw ) throws XMLRepresentationException {
		this.schema = schema;
		root = getRootElement(raw);
		checkDocType( root );
	}
	
	private void checkDocType(Element root) throws XMLRepresentationException {
		if (!DOCUMENT_TYPE.equals(root.getName())) {
			throw new XMLRepresentationException(
				"didn't get expected configuration document "
					+ DOCUMENT_TYPE
					+ " instead got: "
					+ root.getName());
		}
	}

	protected Element getRoot() {
		return root;
	}

	Enumeration objects() {
		return new ObjectEnumeration();
	}

	private class ObjectEnumeration implements Enumeration {
		int i = 0;

		private void scan() {
			while( i < root.getChildCount() ) {
				if( root.getType( i ) == Node.ELEMENT ) {
					return;
				} else {
					i++;
				}
			}			
		}

		public boolean hasMoreElements() {
			scan();
			return i < root.getChildCount();
		}

		public Object nextElement() {
			scan();
			Element e = root.getElement( i++ );
			try {
				return new ConfigObject( schema, e );
			} catch (XMLRepresentationException e1) {
				critical( e1 );
				return null;
			}
		}

		private void critical(Exception e) {
			Log.critical("ObjectEnumeration", e );
		}
	}
}
