/*
 * Created on Sep 5, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.messaging;

import java.io.IOException;

import com.instant802.cldc.DataPath.GenericRecord;
import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.clustermastering.AgentNotRunningException;
import com.instant802.cldc.clustermastering.InvalidTimingException;
import com.instant802.cldc.clustermembership.MembershipAgent;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class Message extends GenericRecord {

	private static final String SENDER_ID = "sender_id";
	private static final String SENDER_IP = "sender_ip";
	private static final String CLUSTER_ID = "cluster_id";
	private static final String SEQUENCE_NUMBER = "sequence_number";
	private static final String PROTOCOL_VERSION = "protocol_version";
	private static final String PROTOCOL_REVISION = "protocol_revision";
	private static final String MODEL_NUMBER = "model_number";

	static int PROTOCOL_VERSION_NO = 1;
	static int PROTOCOL_REVISION_NO = 14;
	static String SYSTEM_MODEL_NUMBER = "unspecified instant802";

	private static int sequence = 0;
	
	private ReplyMechanism toReply;

	private synchronized static int sequenceNumber() {
		return sequence++;
	}

	protected Message( ReplyMechanism toReply, XMLRecord record ) throws XMLRepresentationException {
		this.toReply = toReply;
		makeFromXMLRecord( record );
	}

	protected Message( final OrchestratorNode node  ) throws ElementNotFoundException {
		intField( PROTOCOL_VERSION, PROTOCOL_VERSION_NO );
		intField( PROTOCOL_REVISION, PROTOCOL_REVISION_NO );
		field( MODEL_NUMBER, SYSTEM_MODEL_NUMBER );
		field( SENDER_ID ,node.getNodeId() );
		field( SENDER_IP, node.adminInterface().ipAddress() );
		field( CLUSTER_ID, node.masterCriteria().getClusterID() );
		intField( SEQUENCE_NUMBER, sequenceNumber() );
	}
				
	public static void setModelNumber( String number ) {
		SYSTEM_MODEL_NUMBER = number;
	}

	public int protocolVersion() {
		return intField( PROTOCOL_VERSION );
	}
	
	public int protocolRevision() {
		return intField( PROTOCOL_REVISION );
	}

	public String modelNumber() {
		return field( MODEL_NUMBER );
	}
				
	public String getClusterId() {
		return field( CLUSTER_ID );
	}
	
	public String getSenderId() {
		return field( SENDER_ID );
	}

	public String getSenderIp() {
		return field( SENDER_IP );
	}

	/**
	 * This method is called when the message is received.  It should encode the actual
	 * behaviour of what the node does when it receives such a message.
	 */
	public abstract void onReceipt(final OrchestratorNode node ) throws ElementNotFoundException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException, InvalidTimingException, SocketException, AgentNotRunningException;

	public void sendReply(OrchestratorNode node, Message message) throws IOException, SocketException {
		if (Log.DEBUG) debug("sending reply");
		toReply.sendMessage( message, getSenderIp() );
		if (Log.DEBUG) debug("reply sent");
	}

	public boolean fromMember() {
		return ! getClusterId().equals( MembershipAgent.NON_MEMBER );
	}

	public void checkProtocolVersion() throws BadMessageException {
		if (Log.DEBUG) debug("checking protocol version");
		if( protocolVersion() != PROTOCOL_VERSION_NO ) {
			throw new ProtocolVersionException( "message is protocol version "+protocolVersion()+" expecting "+PROTOCOL_VERSION_NO );
		}
		if( protocolRevision() != PROTOCOL_REVISION_NO ) {
			throw new ProtocolRevisionException( "message is protocol revision "+protocolRevision()+" expecting "+PROTOCOL_REVISION_NO );
		}

		if( ! modelNumber().equals( SYSTEM_MODEL_NUMBER ) ) {
		    throw new ModelNumberException( "model number is "+modelNumber()+" expecting "+SYSTEM_MODEL_NUMBER );
		}
	}
}
