/*
 * Created on Sep 5, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.messaging;

import java.io.IOException;

import sun.rmi.server.UnicastRef;

import com.instant802.cldc.net.Datagram;
import com.instant802.cldc.net.DatagramSocket;
import com.instant802.cldc.net.UDPDatagramSocket;
import com.instant802.cldc.net.UnixDatagramSocket;

import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.clustermastering.AgentNotRunningException;
import com.instant802.cldc.clustermastering.InvalidTimingException;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.ControlledQueue;
import com.instant802.cldc.util.ControlledThread;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.QueueClosedException;
import com.instant802.cldc.util.ThreadCounter;
import com.instant802.cldc.net.SocketTimeoutException;

/**
 * @author robin, alanl
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class MessageReceiver
        extends Thread
        implements ControlledThread, ReplyMechanism
{
		private static final boolean LOCAL = true;
		private static final String RECEIVER_SOCKET = "orchestrator-incoming";
		private static final String SENDER_SOCKET = "orchestrator-outgoing";
        private static final String TYPE = "MessageReceiver";
        private static final int BUFFERSIZE = 8192;
        private boolean running = true;
        final OrchestratorNode node;
        private final DatagramSocket socket;
        private final MessageExecutor executor;
		final ControlledQueue messageQueue = new ControlledQueue();

        public MessageReceiver(OrchestratorNode node) throws IOException
        {
                this.node = node;
                if( LOCAL ) {
                	this.socket = createUnixSocket();
                } else {
                	this.socket = createUDPSocket();
                }
                this.executor = new MessageExecutor();
                ThreadCounter.register(this);
                this.setPriority( MAX_PRIORITY );
        }

        public void finalize()
        {
                ThreadCounter.deregister(this);
        }

        /**
         * @param datagram
         * @return
         */
        public boolean handleDatagram(Datagram datagram)
        {
                try
                {
                        XMLRecord record = new XMLRecord(datagram.getData());
                        debug("created xml record " + record.typeName());
                        return handleXMLRecord(record);
                }
                catch (XMLRepresentationException e)
                {
                        Log.warning(e);
                }
                catch (MessageNotUnderstoodException e)
                {
                        Log.warning(e);
                }
                catch (RuntimeException e)
                {
                        Log.emergency("MessageReceiver", e);
                }
                return false;
        }

        private boolean handleXMLRecord(XMLRecord record)
                throws XMLRepresentationException, MessageNotUnderstoodException
        {
                Message message;
                try
                {
                        message = MessageClassifier.classifyMessage(this, record);
                        if (message != null)
                        {
                                process(message);
                                debug("message handled");
                                return true;
                        }
                        else
                        {
                                return false;
                        }
                }
                catch (BadMessageException e)
                {
                        error(e);
                        return false;
                }
        }

        private void process(Message message)
        {
                debug("processing message " + message);
                debug("node is " + node);
                if (null != node)
                        debug("nodeId is " + node.getNodeId());
                if (!message.getSenderId().equals(node.getNodeId()))
                {
                        Log.info("received message " + message);
						try {
							messageQueue.enqueue( message );
							debug("message enqueued");
						} catch (QueueClosedException e) {
							debug("message queue was closed");
							running = false;							
						}
                }
                else
                {
                        debug("message was sent by us");
                }
        }

        private void debug(String message)
        {
                Log.debug("MessageReceiver", message);
        }

        private UnixDatagramSocket createUnixSocket() throws IOException
		{
            try
            {
            		UnixDatagramSocket socket = new UnixDatagramSocket( RECEIVER_SOCKET, SENDER_SOCKET );
            		return socket;
            }
            catch (SocketException se)
            {
                    throw new IOException(se.getMessage());
            }
		
		}
        
        private UDPDatagramSocket createUDPSocket() throws IOException
        {
                try
                {
                		UDPDatagramSocket socket = new UDPDatagramSocket( node.getMessagePort() );
                		socket.bindToInterface( node.adminInterface() );
                		return socket;
                }
                catch (SocketException se)
                {
                        throw new IOException(se.getMessage());
                }
        }

		public void run() {
			debug("starting");
			try {
				executor.start();
				while (running) {
					try {
						debug("about to get datagram");
						getDatagram();
						debug("handled datagram");
					} catch (SocketTimeoutException toe) {
						debug("timed out waiting for datagram");
						try {
							Thread.sleep(10);
						} catch (Exception _) {
						}
					} catch (SocketException e) {
						error(e);
					}
				}
			} catch (Exception e) {
				emergency(e);
			} finally {
				executor.waitForStop();
				socket.close();
				debug("stopped");
			}
		}

        private void emergency(Exception e) {
        	Log.emergency( e );
		}

		/**
         * @param e
         */
        private void error(Exception e)
        {
                Log.error(TYPE, e);
        }

        private void getDatagram() throws SocketException
        {
                byte buffer[] = new byte[BUFFERSIZE];
                Datagram p = socket.receive(buffer);
                if (null != p)
                {
                        byte[] ba = p.getData();
                        if (null == ba || ba.length <= 0)
                        {
                                debug("packet was empty - yielding");
                                try {Thread.sleep(10); } catch (Exception _) {}
                        }
                        else
                        {
                                receiveDatagram(p);
                        }
                }
        }

        /**
         * @param ba
         */
        private void receiveDatagram(Datagram p)
        {
                if (isXML(p))
                {
                        handleDatagram(p);
                }
                // used to hand the datagram to the legacy requestresponse object here
                // - now we just drop it.
        }

        /**
         * determine whether a datagram contains an XML document.
         * 
         * @param datagram
         *                     the datagram to check.
         * @return true if the datagram contains an XML document.
         */
        private boolean isXML(Datagram datagram)
        {
                String msg = new String(datagram.getData(), 0, datagram.getLength());
                debug("examining received datagram: " + msg);
                boolean isXML = (msg.indexOf("<?xml") != -1);
                debug("datagram is xml? : " + isXML);
                return isXML;
        }

        /**
         * @param bs
         * @param i
         * @param responseAddress
         * @return
         */
        private Datagram makeDatagram(byte[] bs, int i, String responseAddress)
                throws IOException
        {
                Datagram p = new Datagram(bs, (short) i);
                debug(
                        "socket is "
                                + socket
                                + " creating datagram "
                                + responseAddress
                                + ":"
                                + node.getMessagePort());
                p.setAddress(responseAddress);
                p.setPort(node.getMessagePort());
                return p;
        }

        /**
         * @param bs
         * @param i
         * @param responseAddress
         * @return @throws
         *                IOException
         * @deprecated
         */
        /**
         * public Datagram newDatagram(byte[] bs, int i, String responseAddress)
         * throws IOException { return socket.newDatagram(bs, i, responseAddress); }
         */

        public void send(Datagram p) throws IOException
        {
                try
                {
                        socket.send(p);
                }
                catch (SocketException e)
                {
                        throw new IOException(e.getMessage());
                }
        }

        public void waitForStop()
        {
                debug("shutting down");
                running = false;
                debug("waiting for stop");
                try
                {
                        this.join();
                }
                catch (InterruptedException e)
                {
                        error(e);
                }
                debug("done shutting down");
        }

        private class MessageExecutor extends Thread implements ControlledThread
        {
        		private boolean running = true;

                MessageExecutor()
                {
                        ThreadCounter.register(this);
                        debug("created new message executor");
                        this.setPriority( MAX_PRIORITY );
                }

				public void finalize() {
					ThreadCounter.deregister(this);
				}

				public void run() {
					while( running ) {
						Message message;
						try {
							debug("waiting for message from queue");
							message = (Message) messageQueue.dequeue();
							debug("dequeued message to process");
							processMessage( message );
							debug("done processing message");
						} catch (QueueClosedException e) {
							running = false;
						}
					}
				}

				public void processMessage( Message message )
                {
                        debug("processing message");
                        try {
                                message.onReceipt( node );
                        } catch(SocketException e) {
                                error( e );
                        } catch(IOException e) {
                                error( e );
                        } catch(ElementNotFoundException e) {
                                warning( e );
                        } catch (XMLRepresentationException e) {
                                warning( e );
                        } catch (MalformedHTTPMessageException e) {
                                critical( e );
                        } catch (LSQLException e) {
                        	node.handleLSQLException( e );
                        } catch (APConfException e) {
                                emergency( e );
                        } catch (InvalidTimingException e) {
                                critical( e );
                        } catch (AgentNotRunningException e) {
                                emergency( e );
                        }
                        debug("done");
                }
                
                private void emergency(Exception e) {
                        Log.emergency( objectName(), e );
                }

                private void critical(Throwable e)
                {
                        Log.critical(objectName(), e);
                }

                private void warning(Exception e)
                {
                        Log.warning(objectName(), e);
                }

                private void debug(String message)
                {
                        Log.debug(objectName(), message);
                }

                private void error(Exception e)
                {
                        Log.error(objectName(), e);
                }

                private String objectName()
                {
                        return "MessageExecutor";
                }

				public void waitForStop() {
					running = false;
					messageQueue.close();
					try {
						this.join();
					} catch (InterruptedException e) {
						error( e );
					}
				}
        }

        public void sendMessage(Message toSend, String address) throws IOException
        {
                debug("sendMessage " + toSend + " to address " + address);
                byte[] b = toSend.asXMLDocument().getBytes();
                debug("created buffer");
                Datagram d = makeDatagram(b, b.length, address);
                debug("created datagram");
                try
                {
                        socket.send(d);
                }
                catch (SocketException e)
                {
                        throw new IOException(e.getMessage());
                }
                debug("sent message");
        }


}
