/*
 * Created on Aug 30, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.net;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import com.instant802.cldc.util.Log;


/**
 * @author robin
 *
 * Frame simple records with a null character for sending or receiving over sockets.
 * uses null as a terminator without any form of escaping so is useless for any data which
 * might contain nulls.
 */
public class TrivialRecordFramer {

	final Socket socket;
	static final byte TERMINATOR_CHAR = '\0';
	static final byte[] terminator = { TERMINATOR_CHAR };
	final ByteArrayOutputStream in = new ByteArrayOutputStream();
	byte[] buffer;
	int read, next;
	
	public TrivialRecordFramer( Socket socket ) {
		this.socket = socket;
		buffer = new byte[1024];
		read = 0;
		next = 0;
	}

	/**
	 * send the next record over the socket
	 * @param record
	 * @throws SocketException
	 */
	public void sendRecord( byte[] record ) throws SocketException {
//		synchronized( socket ) {
			socket.write( record, record.length );
			socket.write( terminator, terminator.length );
//		}
	}
	
	
	/**
	 * receive the next record from the socket.
	 */
	public byte[] receiveRecord() throws SocketException, IOException {
		//TODO: examine this for efficiency issues if doing performance optimization
		// ALANL: Done -- block reads are much more efficient
//		synchronized( socket ) {
			synchronized (in) {
				try {
					boolean started = false;
					while (true) {
						if (next >= read) {
							read = socket.read(buffer, buffer.length);
							next = 0;
						}
						if (! started) {
							started = true;
						}						
						if (next < read) {
							int tindex;
							for (tindex = next; tindex < read; ++tindex) {
								if (buffer[tindex] == TERMINATOR_CHAR) {
									break;
								}									
							}
							in.write(buffer, next, tindex - next);
							next = tindex + 1; // skip terminator
							if (tindex < read) {
								final byte[] result = in.toByteArray();
								return result;			
							}						
						}
					}

				} finally {
					in.reset();
				}
			}
//		} 
	}
}
