/*
 * Created on Nov 13, 2003
 *
 * (c) 2003 Robin Barooah
 */
package com.instant802.cldc.nodestatus;

import com.instant802.cldc.Activation.Activator;
import com.instant802.cldc.Activation.InvocationMessage;
import com.instant802.cldc.Activation.InvocationResult;
import com.instant802.cldc.Activation.Service;
import com.instant802.cldc.Activation.ServiceInstanceManager;
import com.instant802.cldc.DataPath.GenericRecord;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.ThreadCounter;

/**
* (c) 2003 Instant802 Networks Inc
*/
public class NodeStatusMonitor extends Service {

	private OrchestratorNode node;
	static String TYPE="NodeStatusMonitor";
	private static NodeStatusMonitor instance;
	private static final NodeStatusRecord record = new NodeStatusRecord();

	private NodeStatusMonitor( OrchestratorNode node ) {
		this.node = node;
	}

	public synchronized static NodeStatusMonitor getInstance( OrchestratorNode node ) {
		if( instance == null ) {
			instance = new NodeStatusMonitor( node );
		}
		return instance;
	}

	public static NodeStatusRecord record( OrchestratorNode node ) {
		return record;
	}

	protected boolean doMethod(
		final String method,
		final InvocationMessage message,
		final InvocationResult result)
		throws IllegalStateException {
		if (method.equals("getStatus")) {
			updateRecord();
			result.add( record );
			return true;
		}
		return false;
	}

	private void updateRecord() {
		if (Log.DEBUG) info( "updating status record");
//		record.setMemberCount( node.membershipAgent().memberCount() );

		record.setMemberCount( node.masterCriteria().members() );		
		
		if (Log.DEBUG) info( "updated member count");
		record.setCoordinationMode( node.coordinationAgent().isServer() );
		if (Log.DEBUG) info( "updated coordination mode");
		record.setThreadCount();
		if (Log.DEBUG) info( "updated thread count");
		record.setThreadsUsed( ThreadCounter.getThreadsUsed() );
		if (Log.DEBUG) info("done updating status record");
	}

	private void info( String s ) {
		Log.info( s );
	}

	static class InstanceManager extends ServiceInstanceManager {
		public String typeName() {
			return TYPE;
		}

		/* (non-Javadoc)
		 * @see com.instant802.cldc.Activation.ServiceInstanceManager#getInstance(com.instant802.cldc.cluster.OrchestratorNode, com.instant802.cldc.Activation.Activator)
		 */
		public Service getInstance(
			String name,
			OrchestratorNode node,
			Activator a)
			throws Exception {
			return NodeStatusMonitor.getInstance( node );
		}
	}
}
