/*
 * Created on Sep 17, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.orchestrator;

import com.instant802.cldc.linux.NetInterface;
import com.instant802.cldc.net.UDPDatagramSocket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class AdminInterface extends NetInterface {

	private final String name;
	private final String macAddress;
	private String ipAddress;
	private static final String GENERAL_BROADCAST = "255.255.255.255";
	private UDPDatagramSocket broadcastSocket;
	
	public AdminInterface(String string) {
		if (Log.DEBUG) debug("constructing with name: "+string );
		this.name = string;
		this.ipAddress = waitForIpAddr( name );
		if (Log.DEBUG) debug("ip address is: "+ipAddress );
		this.macAddress = waitForMacAddr( name );
		if (Log.DEBUG) debug("hardware interface is: "+macAddress);
	
	}

	public String getDeviceName() {
	    return this.name;
	}

	/**
	 * @param string
	 */
	private void debug(String string) {
		Log.debug("AdminInterface", string );
	}

	public boolean addressChanged() {
		String newAddress = waitForIpAddr( name );
		if( ! newAddress.equals( ipAddress  ) ) {
			ipAddress = newAddress;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Block until an ip address is obtained
	 */
	private String waitForIpAddr( String name )
	{
		String found = null;
		int count = 0;
		while( true ) {
			if (Log.DEBUG) debug("trying to get address for: "+name );
			found = getIPAddress( name );
			if( found != null ) {
				return found;
			} else {
				count ++;
				if( count > 3 ) {
					count = 0;
					pause();
				} else {
					try {Thread.sleep(10); } catch (Exception _) {}
				}
			}
		}
	}

	private String waitForMacAddr( String name )
	{
		String found = null;
		int count = 0;
		while( true ) {
			if (Log.DEBUG) debug("trying to get address for: "+name );
			found = getHardwareAddress( name );
			if( found != null ) {
				return found;
			} else {
				count ++;
				if( count > 3 ) {
					count = 0;
					pause();
				} else {
					try {Thread.sleep(10); } catch (Exception _) {}
				}
			}
		}
	}
	
	private synchronized void pause() {
		try {
			if (Log.DEBUG) debug("waiting for 3 seconds");
			wait(3000);
		} catch (InterruptedException ie) {
			error(ie);
		}
	}

	private void error(Exception ie) {
		Log.error("AdminInterface", ie );
	}

	public String macAddress() {
		return macAddress;
	}

	public synchronized String ipAddress() {
		if( ipAddress == null ) {
			waitForIpAddr( name );
		}
		return ipAddress;
	}

	public String netmask() {
		return getNetmask( name );
	}
	
//	private byte[] aton( String address ) {
//		byte[] ipv4 = new byte[4];
//		for( int i = 0; i < 4; i++ ) {
//			String octet = dotSegment( address, i );
//			ipv4[i] = (byte) Integer.parseInt( octet );
//		}
//		return ipv4;
//	}
//	
//	private String dotSegment(String address, final int segment ) {
//		address = address.trim();
//		int start = 0;
//		for( int  i = 0; i < segment; i ++ ) {
//			start = address.indexOf('.', start  ) + 1;
//		}
//		int end = 0;
//		if( segment == 3 ) {
//			end = address.length();
//		} else {
//			end = address.indexOf('.', start );
//		}
//		String octet = address.substring( start, end );
//		return octet;
//	}

//	private String subnetBroadcastAddress() {
//		byte[] netmask = aton( netmask() );
//		byte[] ipAddress = aton( ipAddress() );
//		byte[] broadcastbits = not( netmask );
//		return ntoa( or( ipAddress, broadcastbits)  );
//	}
	
//	private String ntoa(byte[] bs) {
//		StringBuffer b = new StringBuffer();
//		for( int i = 0; i < bs.length; i++ ) {
//			if( bs[i] > 0 ) {
//				b.append( bs[i] );
//			} else {
//				b.append( 256 + bs[i] );
//			}
//			if( i < (bs.length - 1) ) {
//				b.append( "." );
//			}
//		}
//		return b.toString();
//	}

	/**
	 * @param ipAddress
	 * @param broadcastbits
	 */
//	private byte[] or(byte[] a, byte[] b) {
//		byte[] result = new byte[ a.length ];
//		for( int i = 0; i < a.length; i++ ) {
//			result[ i ] = (byte) ( b[i] | a[i] );
//		}
//		return result;		
//	}
//
//	private byte[] not(byte[] a) {
//		byte[] result = new byte[ a.length ];
//		for( int i = 0; i < a.length; i++ ) {
//			result[ i ] = (byte) ~ a[i];
//		}
//		return result;
//	}

	private synchronized void setupBroadcastSocket() throws SocketException {
		if( broadcastSocket == null ) {
			broadcastSocket = new UDPDatagramSocket();
			broadcastSocket.enableBroadcast();
		}
	}
	
	public void broadcast(byte[] b, int port ) throws SocketException {
		setupBroadcastSocket();
//		String address = subnetBroadcastAddress();
		String address = GENERAL_BROADCAST;
		broadcastSocket.sendTo( b, address, port ); 
	}
	
	public synchronized void shutdown() {
		if( broadcastSocket != null ) {
			broadcastSocket.close();
			broadcastSocket = null;
		}
	}	
}
