/*
 * Created on Apr 20, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;

import java.util.Hashtable;

import com.instant802.cldc.ssl.OpenSSL;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 *
 * CHAP/MD5. The secret is the PAP/CHAP password found in radiusserver.getUserPassword()
 */
public class CHAPMethod
{

	private RadiusServer server;
	private byte[] user;
	private byte[] secret;

	public final static int CHAP_CHALLENGE = 1, CHAP_RESPONSE = 2;

	// hash table of challenges sent to users. Since this radius server is embedded
	// we needn't bother ourselves with tracking addresses of NASs.
	private static Hashtable userchallenges = new Hashtable();

	public CHAPMethod(RadiusServer server, byte[] user, byte[] secret)
	{
		this.server = server;
		this.user = user;
		this.secret = secret;
	}

	private byte[] generateChapChallenge(byte[] user)
	{
		byte[] challenge = new byte[16];
		long ts = System.currentTimeMillis();
		byte[] buf = new byte[8 + user.length];

		System.arraycopy(user, 0, buf, 0, user.length);

		for (int i = 0; i < 8; ++i)
		{
			buf[user.length + i] = (byte) (0xff & (ts >> (8 * i)));
		}
		new OpenSSL().md5(buf, buf.length, challenge);
		userchallenges.put(new String(user), challenge);
		return challenge;
	}

	/**
	 * Much to my surprise the CHAP packet is embedded in EAP as simply [value-size|chap value]
	 * and not [size|chap packet]. See rfc2284 section 3.4 and read _very_ carefully. This
	 * is an MD5 thing (cf PEAP/MSChapV2)
	 * @param user
	 * @param id
	 * @return
	 */
	private byte[] createChallenge()
	{
		byte[] challenge = generateChapChallenge(user);
		byte[] sysname = server.getSystemName();
		byte[] chappacket = new byte[1 + challenge.length];

		chappacket[0] = (byte) (0xff & challenge.length);
		System.arraycopy(challenge, 0, chappacket, 1, challenge.length);
		return chappacket;
	}

	private boolean verifyChallenge(byte[] response, byte id)
	{
		byte[] challenge = (byte[]) userchallenges.get(new String(user));
		byte[] secret = (byte[]) server.getPlaintextPassword(user);

		if (null == challenge || null == secret)
		{
			return false;
		}
		if (Log.DEBUG) Log.debug("CHAP: challenge="+ Utils.hex(challenge) + " secret=" + Utils.hex(secret));
		byte[] grist = new byte[1 + secret.length + challenge.length];
		byte[] hash = new byte[16];
		grist[0] = id;
		System.arraycopy(secret, 0, grist, 1, secret.length);
		System.arraycopy(challenge, 0, grist, 1 + secret.length, challenge.length);
		new OpenSSL().md5(grist, grist.length, hash);
		return Utils.compareBytes(response, 1, hash, 0);
	}

	/**
	* @param eap
	* @param rp
	* @param secret
	*/
	public int doChapResponse(EAPPacket eap, RadiusResponse resp)
	{
		byte[] response = eap.getTypeData();
		EAPPacket eapresp;

		eapresp = doChapResponse(response, eap.getId());
		eapresp.embedInRadiusResponse(resp);
		return eapresp.getCode();
	}


	public EAPPacket doChapResponse(byte[] response, byte eapid)
	{
		String uname = ((null != user) ? new String(user, 0, user.length) : "<Unknown>");
		int eapcode;

		if (verifyChallenge(response, eapid))
		{
			eapcode = EAPHandler.EAP_CODE_SUCCESS;
			if (Log.DEBUG) Log.debug("User " + uname + " passed MD5 challenge");

		}
		else
		{
			eapcode = EAPHandler.EAP_CODE_FAILURE;
			if (Log.DEBUG) Log.debug("User " + uname + " failed MD5 challenge");
		}
		return new EAPPacket(eapcode, (byte)(eapid + 1));
	}
	
	public int doChapChallenge(EAPPacket eap, RadiusResponse resp)
	{
		byte[] chapchallenge = createChallenge();
		if (Log.DEBUG) Log.debug(
			"Issuing MD5 challenge to user  "
				+ ((null != user) ? new String(user, 0, user.length) : "<Unknown>"));

		EAPPacket eapreply =
			new EAPPacket(
				EAPHandler.EAP_CODE_REQUEST,
				EAPHandler.EAP_TYPE_MD5,
				(byte)(eap.getId() + 1),
				chapchallenge);
		eapreply.embedInRadiusResponse(resp);
		return EAPHandler.EAP_CODE_REQUEST;
	}
	
	// For MD5 through PEAP
	public EAPPacket doChapChallenge(byte eapid) {
		byte[] chapchallenge = createChallenge();
		if (Log.DEBUG) Log.debug(
			"Issuing MD5 challenge to user  "
				+ ((null != user) ? new String(user, 0, user.length) : "<Unknown>"));

		EAPPacket eapreply =
			new EAPPacket(
				EAPHandler.EAP_CODE_REQUEST,
				EAPHandler.EAP_TYPE_MD5,
				(byte)(eapid + 1),
				chapchallenge);
		return eapreply;				
	}

}
