/*
 * Created on May 11, 2004
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package com.instant802.cldc.san;

import java.util.Vector;

import com.instant802.cldc.hostapd.NeighboringAP;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class APSanGraph extends Graph implements SANGraph
{

	int[] lastchannels;
	int channels;
	int[] channelbag;
	Vector manageableAPs;
	int manageables_size;
	Vector locked_aps;

	public APSanGraph(AP[] aps, Vector locked_aps, int[] channelbag)
	{
		super(aps.length);
		AP ap;

		this.channels = channelbag.length;
		this.channelbag = channelbag;
		this.locked_aps = locked_aps;
		setNodes(aps);
		this.size = aps.length;
		if (Log.DEBUG) Log.debug("Wireless neigborhood has " + size + " Access Points");
		manageableAPs = new Vector();
		for (int i = 0; i < size; ++i)
		{
			ap = aps[i];
			if ((ap.getIPAddress() != null) && inBand(ap) && (!isLocked(ap)))
			{
				manageableAPs.addElement(ap);
			}
		}
		manageables_size = manageableAPs.size();
		lastchannels = new int[manageables_size];
		if (Log.DEBUG) Log.debug(manageables_size + " Access Points are manageable");
	}

	// Is this AP in the same band as the planning process ?? Naive
	// approach is tricky since some regularity domains have channel numbers which
	// spill between various bands.

	private boolean inBand(AP ap)
	{	
		return ap.getBand().equals(currentBand());
	}


	public String currentBand()
	{
		return AP.getBand(channelbag);
	}
	
	public boolean isLocked(AP ap)
	{
		return locked_aps.contains(ap.getIPAddress());
	}

	public boolean isManageable(AP ap)
	{
		return manageableAPs.contains(ap);
	}

	public AP getAP(int index)
	{
		return (AP) nodes[index];
	}

	public float energy() throws EnergyCalculationException
	{
		float energy = 0.0f;
		AP n1, n2;

		for (int i = 0; i < size; ++i)
			for (int j = 0; j < size; ++j)
			{
				if (i == j || 0.0f == graph[i][j])
					continue;
				n1 = (AP) getNode(i);
				n2 = (AP) getNode(j);
				if (n1.getChannel() == n2.getChannel())
					energy += graph[i][j];
			}
		return energy;
	}

	public void init() throws GraphInitFailureException
	{
		if (manageables_size == 0)
			throw new GraphInitFailureException();
	}

	// Choose a manageable node at random and change its channel
	public void perturb() throws Exception
	{
		int index;
		saveState();

		index = rand.nextInt(manageables_size);
		((AP) manageableAPs.elementAt(index)).setChannel(
			channelbag[rand.nextInt(channels)]);
	}

	private void saveState() throws Exception
	{
		for (int i = 0; i < manageables_size; ++i)
			lastchannels[i] = ((AP) manageableAPs.elementAt(i)).getChannel();
	}

	public void restoreState() throws Exception
	{
		for (int i = 0; i < manageables_size; ++i)
			 ((AP) manageableAPs.elementAt(i)).setChannel(lastchannels[i]);
	}

	public void dump()
	{

		for (int i = 0; i < size; ++i)
		{
			StringBuffer sb = new StringBuffer(getAP(i).getMACAddress() + ": ");
			for (int j = 0; j < size; ++j)
			{
				if (graph[i][j] > 0)
					sb = sb.append("    ").append(getAP(j).getMACAddress());
			}
			if (Log.DEBUG) Log.debug(sb.toString());
		}
	}
}
