/*
 * Created on Apr 23, 2003
 * 
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.ssl;

import com.instant802.cldc.radius.Utils;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 * 
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SSLMem extends SSL
{

	private BIO inBio, outBio, filterbio;

	/**
	 * @param SSL_CONTEXT_MODE_SERVER
	 */

	public SSLMem(SSLSessionContext ctx)
	{
		super(ctx);
		inBio = new BIO();
		inBio.createBIOSMem();
		outBio = new BIO();
		outBio.createBIOSMem();
		setBIO(ssl, inBio.getCPointer(), outBio.getCPointer());
		filterbio = new BIO();
		filterbio.createBIOFSSL(ssl);
	}

	private native void setBIO(int sslptr, int inbioptr, int outptr);

	/**
	 * Both read and writes to the SSL object can result in data being sent to
	 * the Peer - use this method to get those bytes and then arrange to send
	 * them to the peer. e.g. To respond to a peer by sending bytes back:
	 * 
	 * <pre>
	 *  sslMem = new SSLMem(ctx); .......... fill buf with data to be sent sslMem.write(buf); while ( sslMem.pending() > 0) { bytes = readPeerChannel(sendbuffer); ... process sendbuffer ... }
	 * </pre>
	 * 
	 * Reading is slightly more complex since the read might create data which
	 * needs to be sent back (e.g. starting the TLS handshake) so we send these
	 * back before processing our read buffer:
	 * 
	 * <pre>
	 *  sslMem = new SSLMem(ctx); ... get encrypted bytes from source into datain // read our end through SSL bytes = sslMem.read(datain, decryptedbuffer); while ( sslMem.pending() > 0) { bytes = readPeerChannel(sendbuffer); ... process sendbuffer ... } // now process bytes in decrypted buffer 
	 * </pre>
	 * 
	 * @param buffer
	 * @return nuber bytes to send
	 */
	public int readPeerChannel(byte[] buffer)
	{
		return outBio.read(buffer, buffer.length);
	}

	public int pending()
	{
		return outBio.pending();
	}
	/**
	 * Data is 'read' from the source by pulling it through the SSL Filter
	 * 
	 * @param datain
	 *                     Buffer of data coming in to be passed through SSL fileter
	 * @param dataout
	 *                     Buffer to be filled as it is read from SSL filter. Up to user
	 *                     to make sure this is large enough
	 * @return Number of significant bytes in dataout
	 * @author alan
	 *  
	 */
	public int read(byte[] datain, byte[] dataout)
	{
		if (Log.DEBUG) Log.debug("Pulling data through SSL filter");
		inBio.write(datain, datain.length);
		int ret = filterbio.read(dataout, dataout.length);
		if (Log.DEBUG) Log.debug(" Pulled " + ret + " bytes through filter");
		return ret;
	}

	/**
	 * Data is 'written' by pushing through the SSL filter using this method,
	 * then using readPeerChannel to get the bytes to send the encrypted bytes
	 * back to the peer.
	 * 
	 * @param dataout
	 *                     Bytes to be encrypted.
	 */
	public void write(byte[] dataout)
	{
		if (Log.DEBUG) Log.debug("PEAP Inner write: " + Utils.hex(dataout, dataout.length));
		filterbio.write(dataout, dataout.length);
	}
}