/*
 * Created on Sep 28, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.transfer;


import java.io.IOException;

import com.instant802.cldc.ssl.SSLSessionContext;
import com.instant802.cldc.DataPath.XMLConfigRecord;
import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.coordination.ClientAgent;
import com.instant802.cldc.coordination.CoordinatorServer;
import com.instant802.cldc.deviceconfiguration.Schema;
import com.instant802.cldc.linux.NetInterface;
import com.instant802.cldc.messaging.Message;
import com.instant802.cldc.messaging.ReplyMechanism;
import com.instant802.cldc.net.SSLServerSocket;
import com.instant802.cldc.net.SSLSocket;
import com.instant802.cldc.net.Socket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.SocketTimeoutException;
import com.instant802.cldc.net.TrivialRecordFramer;
import com.instant802.cldc.util.Log;


/**
 * @author alan
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public abstract class TransferBase implements ReplyMechanism
{
	private static final String TYPE = "TransferBase";
	protected static int SOCKET_TIMEOUT = 15000;
	int port;
	SSLServerSocket ssocket = null;
	Socket socket;
	SSLSessionContext ctx;
	private TrivialRecordFramer framer;
	private final Schema schema;
	
	public TransferBase(Schema schema, SSLSessionContext ctx) throws SocketException
	{
		this.schema = schema;
		this.ctx = ctx;
		socket = new SSLSocket(ctx);
		setTimeout( SOCKET_TIMEOUT );
		framer = new TrivialRecordFramer( socket );
	}

	protected void setTimeout( int timeout ) {
		if (Log.DEBUG) debug("setting socket timeout");
		Socket tsocket = socket;
		if( tsocket != null ) {
			tsocket.setTimeout( timeout );
		}
		if (Log.DEBUG) debug("done setting socket timeout");
	}
	
	public TransferBase(Schema schema, Socket sock)
	{
		this.schema = schema;
		socket = sock;
		socket.setTimeout(SOCKET_TIMEOUT);
		framer = new TrivialRecordFramer( socket );
	}

	public TransferBase(Schema schema, SSLSessionContext ctx, int port, NetInterface iface ) throws SocketException 
	{
		this.schema = schema;
		this.port = port;
		ssocket = new SSLServerSocket(ctx);
		ssocket.bindToInterface( iface );
		ssocket.setTimeout(SOCKET_TIMEOUT);
		this.ctx = ctx;
		ssocket.listen(port);
		if (Log.DEBUG) debug("done constructing");
	}

	private void debug(String string) {
		if (Log.DEBUG) Log.debug( TYPE, string );
	}

	public synchronized void send( Message msg) throws SocketException
	{
		StringBuffer message = msg.asXMLStringBufferDocument();
		if (Log.DEBUG) debug("framer has send a record of size "+message.length());
		framer.sendRecord( message.getBytes() );
	} 

	public synchronized void send( StringBuffer message ) throws SocketException
	{
		if (Log.DEBUG) debug("framer has sent message of size "+message.length());
		framer.sendRecord( message.getBytes() );
	} 

	public ClientAgent accept( CoordinatorServer server ) throws SocketTimeoutException, SocketException
	{
			if (Log.DEBUG) debug("waiting for connection...");
			Socket sock = ssocket.accept();
			if (Log.DEBUG) debug("received connection...");
			return new ClientAgent( server, schema, sock);
	}

	public XMLRecord receive() throws SocketException, IOException, XMLRepresentationException
	{
		byte[] buf = framer.receiveRecord();
		if (Log.DEBUG) debug("framer has returned a record of size "+buf.length);
		try{
			//TODO: gerridofthis
			return new XMLConfigRecord( schema,  buf );
		} catch ( Exception e ) {
			error( e );
			throw new IOException( e.getMessage() );
		}
	}

	public void connect(String host, int port) throws SocketException
	{
		if (Log.DEBUG) debug("attempting connection to "+host+" on port "+port );
		socket.connect(host, port);
		if (Log.DEBUG) debug("connected.");
	}

	public synchronized void close()
	{
		if (null != ssocket) {
			if (Log.DEBUG) debug("closing server socket");
			try { ssocket.close(); 	} catch (Exception e1) {
				error( e1 );
			}
			ssocket = null;
		}
		if (null != socket) {
			if (Log.DEBUG) debug("closing stream socket");
			try { socket.close(); } catch (Exception e2) {
				error( e2 );
			}
			socket = null;
		}
		if (Log.DEBUG) debug("done closing sockets");
	}

	/**
	 * @param e1
	 */
	private void error(Exception e1) {
		Log.error( TYPE , e1 );
	}

	public void sendMessage(Message message, String address) throws IOException, SocketException {
		send( message );
	}
}
