#!/bin/sh

#
# Usage:
#
# 1) portal.sh init
#
# 2) portal.sh init2 <guest bridge ip address>
#
# 3) portal.sh renew <old ip address> <new ip address>
#
# 4) portal.sh auth <mac address>
#
# 5) portal.sh deny <mac address>
#
# 6) portal.sh denyall
#
# 7) portal.sh term
#


PID_FILE=/var/run/portal_dhcpc.pid
IPTABLES=/sbin/iptables
EBTABLES=/sbin/ebtables
NAT="${IPTABLES} -t nat -A PREROUTING"

case "$1" in
    init)
	#
	# Disable routing on this box (prevents traffic from the guest
	# interface ever getting to the internal one via this box)
	#
	echo "0" > /proc/sys/net/ipv4/ip_forward ;
	#
	# Flush the iptables we plan to use, then allow free access to
	# the internal bridge
	#
	${IPTABLES} -t nat -F PREROUTING ;
	${EBTABLES} -t broute -F BROUTING ;
	/sbin/udhcpc -l /var/udhcpc.lease.brguest --foreground -i brguest \
		-s /home/portal/dhcpc.script &
	echo $! > ${PID_FILE}
	;;
    init2)

	IP=$2

	#
	# Flush the iptables we plan to use, then allow free access to
	# the internal bridge
	#
	${IPTABLES} -t nat -F PREROUTING ;
	${EBTABLES} -t broute -F BROUTING ;

	#
	# Allow any packets not from wlan0guest interface to pass
	#
	${EBTABLES} -t broute -A BROUTING -i ! wlan0guest -j ACCEPT ;

	#
	# Rules of allowing authorised packets to pass to be inserted here
	#

	# Allow ARP packets to pass
	${EBTABLES} -t broute -A BROUTING -p 0x0806 -j ACCEPT ;
	# Allow DNS packets to pass
	${EBTABLES} -t broute -A BROUTING -p 0x0800 --ip-proto 17 --ip-dport 53 -j ACCEPT ;
	# Allow DHCP packets to pass
	${EBTABLES} -t broute -A BROUTING -p 0x0800 --ip-proto 17 --ip-dport 67 -j ACCEPT ;
	${EBTABLES} -t broute -A BROUTING -p 0x0800 --ip-proto 17 --ip-dport 68 -j ACCEPT ;

	#
	# Redirect all packets to the MAC of the brguest
	#
	MAC=`ifconfig brguest|extract 5`
	${EBTABLES} -t broute -A BROUTING -j dnat --to-dst $MAC --dnat-target DROP ;

	#
	# Allow all packets not from brguest to pass
	#
	${NAT} -i ! wlan0guest -j ACCEPT ;

	#
	# Redirect to port 8080
	${IPTABLES} -t nat -A PREROUTING -p tcp --dport 80 -j DNAT --to-destination $IP:8080;

	#
	# Drop any other packets???
	#
	${NAT} -j ACCEPT ;
	;;

    auth)
	${EBTABLES} -t broute -I BROUTING 2 -s $2 -j ACCEPT
	;;

    deny)
	${EBTABLES} -t broute -D BROUTING -s $2 -j ACCEPT
	;;

    denyall)
	;;

    term)
	#
	# Kill the DHCP client (don't ask about the strange steps; msh
	# does not support backticks)
	#
	if [ -f ${PID_FILE} ]; then
		echo -n "kill -9 " > /var/tmp/kill
		cat ${PID_FILE} >> /var/tmp/kill
		/bin/sh  /var/tmp/kill
		rm ${PID_FILE}
		rm /var/tmp/kill
	fi
	#
	# Flush the tables
	#
	${IPTABLES} -t nat -F PREROUTING ;
	${EBTABLES} -t broute -F BROUTING ;
	;;

esac

exit 0
